/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * ClosedCode.java
 *
 *
 */

package uiuc.Jumbo.Compiler;

import java.io.*;
import java.util.*;
import uiuc.Jumbo.Jaemus.*;
import uiuc.Jumbo.Util.*;

/**
 * An interface for all pieces that make up a classfile that has no holes.
 * @version      1.0,  2 Oct 2001
 * @author       Lars Clausen
 *
*/

public final class ClosedCode {
    /* This is basically a tuple */
    public final Instr bytecode;
    public final Environment env;
    public final Type type;
    public final ConstVal constval;
    /*    private MonoList definitions;*/
    MonoList writables;

    public ClosedCode(Instr bytecode, Environment env, Type type) {
        this.bytecode = bytecode;
        this.env = env;
        this.type = type;
        this.constval = null;
    }
    public ClosedCode(Instr bytecode, Type type, Environment env) {
        this(bytecode, env, type);
    }
    public ClosedCode(Instr bytecode, Environment env, Type type, ConstVal v) {
        this.bytecode = bytecode;
        this.env = env;
        this.type = type;
        this.constval = v;
    }
    /*
    private ClosedCode addInfo(ClosedCode retvals) {
        if (definitions == null) {
            definitions = retvals.definitions;
        } else {
            definitions.addAll(retvals.definitions);
        }
        return this;
    }
    private ClosedCode addInfoList(MonoList retvallist) {
        Iterator i = retvallist.iterator();
        if (definitions == null) {
            if (retvallist.size() == 0) return this;
            definitions = ((ClosedCode)i.next()).definitions;
        }
        while (i.hasNext()) {
            definitions.add((ClosedCode)i.next());
        }
        return this;
    }
    */
    public void writearray() {
        for (Iterator i = writables.iterator(); i.hasNext(); ) {
            ClassInfo c = (ClassInfo)i.next(); 
            try {
                c.writeArray();
            } catch (IOException err) {
                throw new Error("Couldn't write class array: "+err);
            }
        }
    }
    public void write() {
        for (Iterator i = writables.iterator(); i.hasNext(); ) {
            ClassInfo c = (ClassInfo)i.next(); 
            try {
                c.writeClassFile();
            } catch (IOException err) {
                throw new Error("Couldn't write class file: "+err);
            }
        }
    }
    public ClosedCode addWritable(ClassInfo c) {
        if (writables == null) writables = new ArrayMonoList();
        writables.add(c);
        return this;
    }
    public ClosedCode addWritable(ClosedCode c) {
        if (c.writables == null) return this;
        if (writables == null) {
            writables = c.writables;
            return this;
        }
        writables.addAll(c.writables);
        return this;
    }
    /*
    private MonoList getInfo() {
        return definitions;
    }
    */
    public boolean isConstant() {
        return constval != null;
    }
    public ConstVal constantValue() {
        return constval;
    }
    public Type type() { return type; }

    public String toString() {
        return "CC<"+type+">";
    }
}

/* End of interface ClosedCode*/
