/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * TestConstants.java
 *
 *
 */

package uiuc.Jumbo.Compiler;

import junit.framework.TestCase;

import uiuc.Jumbo.Jaemus.*;
import uiuc.Jumbo.Compiler.*;
import java.util.*;

public class TestConstants extends TestCase {

    private static final Environment EMPTY_ENV = Environment.emptyEnv();

    public TestConstants(String name) {
        super(name);
    }

    public void testIntegers() {
        checkInt( 1 );
        checkInt( -1 );
        checkInt( 0 );
        checkInt( 100 );
        checkInt( 200 );
        checkInt( -279 );
        checkInt( -1179 );
    }

    public void testIntegerMax() {
        checkInt( Integer.MAX_VALUE - 500000, "MAX - 500000" );
        checkInt( Integer.MAX_VALUE, "MAX" );
    }

    public void testIntegerMin() {
        checkInt( Integer.MIN_VALUE + 500000, "MIN + 500000" );
        checkInt( Integer.MIN_VALUE, "MIN" );
    }

    public void testBytes() {
        checkByte((byte)1);
        checkByte((byte)-1);
        checkByte((byte)100);
        checkByte((byte)-200);
    }

    public void testByteMin() {
        checkByte( Byte.MIN_VALUE, "MIN" );
    }

    public void testByteMax() {
        checkByte( Byte.MAX_VALUE, "MAX" );
    }

    public void testChars() {
        checkChar('a');
        checkChar('Z');
        checkChar('\n');
        checkChar('\043');
    }

    public void testCharMin() {
        checkChar( Character.MIN_VALUE, "MIN" );
    }

    public void testCharMax() {
        checkChar( Character.MAX_VALUE, "MAX" );
    }

    public void testFloats() {
        checkFloat(8000.2342342f);
        checkFloat(1.0f);
        checkFloat(0.0f);
        checkFloat(-1.0f);
        checkFloat(-3000.2342342f);
    }

    public void testFloatMin() {
        checkFloat( Float.MIN_VALUE, "MIN" );
    }

    public void testFloatMax() {
        checkFloat( Float.MAX_VALUE, "MAX" );
    }

    public void testLongs() {
        checkLong(1);
        checkLong(0);
        checkLong(-1);
        checkLong(-39405);
        checkLong(9000);
    }

    public void testLongMin() {
        checkLong( Long.MIN_VALUE, "MIN" );
    }

    public void testLongMax() {
        checkLong( Long.MAX_VALUE, "MAX" );
    }

    public void testDoubles() {
        checkDouble(1.0);
        checkDouble(0.0);
        checkDouble(-1.0);
        checkDouble(-39405.0);
        checkDouble(9000.234234);
    }

    public void testDoubleMin() {
        checkDouble( Double.MIN_VALUE, "MIN" );
    }

    public void testDoubleMax() {
        checkDouble( Double.MAX_VALUE, "MAX" );
    }
    
    public void testAddInt(){
        checkAddInt(5, 3);
        checkAddInt(45, 60);
        checkAddInt(-7, 40);
        checkAddInt(5, -7);
        checkAddInt(25, 25);
    }

    public void testSubIntInt(){
        checkSubIntInt(20, 50);
        checkSubIntInt(5, 20);
        checkSubIntInt(100, -10);
        checkSubIntInt(-40, -40);
        checkSubIntInt(-30, 45);
    }
    
    public void testSubDoubleDouble(){
        checkSubDoubleDouble(5.0, 2.5);
        checkSubDoubleDouble(-5.0, 5.0);
        checkSubDoubleDouble(10.0, -20.10);
        checkSubDoubleDouble(-50.70, -20.70);
        checkSubDoubleDouble(2.5, 5.0);
       
    }

    public void testSubFloatFloat(){
        checkSubFloatFloat(67.54567f, 25.394580f);
        checkSubFloatFloat(23.4543f, 123.4543f);
        checkSubFloatFloat(-323.4353f, 567.234f);
        checkSubFloatFloat(-89.5623f, -200.4536f);
        checkSubFloatFloat(4554.785f, -5000.324f);
    }
        
    public void testSubLongLong(){
        checkSubLongLong(45, 250);
        checkSubLongLong(23, 89);
        checkSubLongLong(-34, -78);
        checkSubLongLong(56, -89); 
        checkSubLongLong(-78, 34);
    }
    
    public void testSubIntLong(){
        checkSubIntLong(7, 6);
        checkSubIntLong(2, 10);
        checkSubIntLong(-23, 45);
        checkSubIntLong(45, -23);
        checkSubIntLong(-23, -45);
    }

    public void testSubFloatLong(){
        checkSubFloatLong(45.67f, 40);
        checkSubFloatLong(2.09f, 5);
        checkSubFloatLong(-25.67f, 45);
        checkSubFloatLong(56.3f, -34);
        checkSubFloatLong(-2.3f, -5);
    }

    public void testSubDoubleLong(){
        checkSubDoubleLong(45.67, 40);
        checkSubDoubleLong(1.03, 7);
        checkSubDoubleLong(-27.68, 38);
        checkSubDoubleLong(45.23, -23);
        checkSubDoubleLong(-1.3, -1);
    }

    public void testSubIntFloat(){
        checkSubIntFloat(34, 23.32f);
        checkSubIntFloat(1, 3.45f);
        checkSubIntFloat(-34, 56.7f);
        checkSubIntFloat(34, -23.34f);
        checkSubIntFloat(-1, -2.3f);
    }

    public void testSubIntDouble(){
        checkSubIntDouble(23, 1.2);
        checkSubIntDouble(2, 3.4);
        checkSubIntDouble(-15, 29.0);
        checkSubIntDouble(18, -5.3);
        checkSubIntDouble(-18, -3.1);
    }

    public void testSubFloatDouble(){
        checkSubFloatDouble(23.45f, 12.3);
        checkSubFloatDouble(12.23f, 56.76);
        checkSubFloatDouble(-23.45f, 87.65);
        checkSubFloatDouble(47.45f, -12.34);
        checkSubFloatDouble(-1.23f, -1.363);
    }

    public void testSubLongInt(){
        checkSubLongInt(23, 12);
        checkSubLongInt(12, 56);
        checkSubLongInt(-23, 88);
        checkSubLongInt(47, -12);
        checkSubLongInt(-1, -1);
    }

    public void testSubLongFloat(){
        checkSubLongFloat(36, 25.45f);
        checkSubLongFloat(3, 5.23f);
        checkSubLongFloat(-23, 67.45f);
        checkSubLongFloat(323, -23.434f);
        checkSubLongFloat(-234, -32.35f);
    }

    public void testSubLongDouble(){
        checkSubLongDouble(36, 24.34);
        checkSubLongDouble(3, 5.45);
        checkSubLongDouble(-23, 453.324);
        checkSubLongDouble(23, -34.45);
        checkSubLongDouble(-234, -54.34);
    }

    public void testSubFloatInt(){
        checkSubFloatInt(23.34f, 12);
        checkSubFloatInt(12.34f, 45);
        checkSubFloatInt(-23.45f, 78);
        checkSubFloatInt(34.324f, -34);
        checkSubFloatInt(-43.45f, -56);
    }

    public void testSubDoubleInt(){
        checkSubDoubleInt(23.34, 12);
        checkSubDoubleInt(12.34, 45);
        checkSubDoubleInt(-23.45, 78);
        checkSubDoubleInt(34.45, -12);
        checkSubDoubleInt(-23.34, -1);
    }

    public void testSubDoubleFloat(){
        checkSubDoubleFloat(23.432, 12.32f);
        checkSubDoubleFloat(1.23, 3.45f);
        checkSubDoubleFloat(-23.45, 52.34f);
        checkSubDoubleFloat(12.3, -1.23f);
        checkSubDoubleFloat(-12.32, -54.45f);
    }


    private void checkInt(int t) {
        checkInt(t, "");
    }

    private void checkInt(int t, String message) {
        ClosedCode c = Constructor.constant(t).eval(EMPTY_ENV);
        assertEquals("integer "+message+": "+t, Instr.genConst(t), c.bytecode);
    }

    private void checkByte(byte t) {
        checkByte(t, "");
    }
    
    private void checkByte(byte t, String message) {
        ClosedCode c = Constructor.constant(t).eval(EMPTY_ENV);
        assertEquals("byte "+message+": "+t, Instr.genConst(t), c.bytecode);
    }

    private void checkChar(char t) {
        checkChar(t, "");
    }

    private void checkChar(char t, String message) {
        ClosedCode c = Constructor.constant(t).eval(EMPTY_ENV);
        assertEquals("char "+message+": "+t, Instr.genConst(t), c.bytecode);
    }

    private void checkFloat(float t) {
        checkFloat(t, "");
    }

    private void checkFloat(float t, String message) {
        ClosedCode c = Constructor.constant(t).eval(EMPTY_ENV);
        assertEquals("float "+message+": "+t, Instr.genConst(t), c.bytecode);
    }

    private void checkLong(long t) {
        checkLong(t, "");
    }

    private void checkLong(long t, String message) {
        ClosedCode c = Constructor.constant(t).eval(EMPTY_ENV);
        assertEquals("long "+message+": "+t, Instr.genConst(t), c.bytecode);
    }

    private void checkDouble(double t) {
        checkDouble(t, "");
    }

    private void checkDouble(double t, String message) {
        ClosedCode c = Constructor.constant(t).eval(EMPTY_ENV);
        assertEquals("double "+message+": "+t, Instr.genConst(t), c.bytecode);
    }
   
    private void checkAddInt(int a, int b){
        checkAddInt(a, b, "");
    }


    private void checkAddInt(int a, int b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.ADD, c1, c2).eval(EMPTY_ENV);
        assertEquals("AddInt" + message +" : "+a+" + "+b,  Instr.genConst(a+b), c.bytecode);
    }

    private void checkSubIntInt(int a , int b){
        checkSubIntInt(a, b, "");
    }

    private void checkSubIntInt(int a, int b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV);
        assertEquals("Subtract Int from Int" + message+ " : " + a + " - "+ b, Instr.genConst(a-b), c.bytecode);
    }

    private void checkSubDoubleDouble(double a, double b){
        checkSubDoubleDouble(a, b, "");
    }


    private void checkSubDoubleDouble(double a, double b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV);
        assertEquals("Subtract Double from Double" + message + " : " + a+ " - " + b, Instr.genConst(a-b), c.bytecode);
    }

    private void checkSubFloatFloat(float a, float b){
        checkSubFloatFloat(a, b, "");
    }

    private void checkSubFloatFloat(float a, float b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Float from Float" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubLongLong(long a, long b){
        checkSubLongLong(a, b, "");
    }

    private void checkSubLongLong(long a, long b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Long from Long" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubIntLong(int a, long b){
        checkSubIntLong(a, b, "");
    }

    private void checkSubIntLong(int a, long b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Long from Int" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubFloatLong(float a, long b){
        checkSubFloatLong(a, b, "");
    }

    private void checkSubFloatLong(float a, long b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Long from Float" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubDoubleLong(double a, long b){
        checkSubDoubleLong(a, b, "");
    }

    private void checkSubDoubleLong(double a, long b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Long from Double" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubIntFloat(int a, float b){
        checkSubIntFloat(a, b, "");
    }

    private void checkSubIntFloat(int a, float b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Float from Int" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubIntDouble(int a, double b){
        checkSubIntDouble(a, b, "");
    }

    private void checkSubIntDouble(int a, double b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Double from Int" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubFloatDouble(float a, double b){
        checkSubFloatDouble(a, b, "");
    }

    private void checkSubFloatDouble(float a, double b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Double from Float" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubLongInt(long a, int b){
        checkSubLongInt(a, b, "");
    }

    private void checkSubLongInt(long a, int b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Int from Long" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubLongFloat(long a, float b){
        checkSubLongFloat(a, b, "");
    }

    private void checkSubLongFloat(long a, float b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Float from Long" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubLongDouble(long a, double b){
        checkSubLongDouble(a, b, "");
    }

    private void checkSubLongDouble(long a, double b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Double from Long" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubFloatInt(float a, int b){
        checkSubFloatInt(a, b, "");
    }

    private void checkSubFloatInt(float a, int b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Int from Float" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubDoubleInt(double a, int b){
        checkSubDoubleInt(a, b, "");
    }

    private void checkSubDoubleInt(double a, int b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Int from Double" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

    private void checkSubDoubleFloat(double a, float b){
        checkSubDoubleFloat(a, b, "");
    }

    private void checkSubDoubleFloat(double a, float b, String message){
        Code c1 = Constructor.constant(a);
        Code c2 = Constructor.constant(b);
        ClosedCode c = Constructor.mathOp(Instr.SUB, c1, c2).eval(EMPTY_ENV); 
        assertEquals("Subtract Float from Double" + message + " : " + a+ " - " + b, Instr.genConst(a - b), c.bytecode);
    }

}




