/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * TestMethods.java
 *
 *
 */

package uiuc.Jumbo.Compiler;

import junit.framework.TestCase;

import uiuc.Jumbo.Jaemus.*;
import uiuc.Jumbo.Util.*;

import java.util.*;

/**
 * Tests of method creation
 * @version      1.0,  1 Nov 2001
 * @author       Lars Clausen
 *
*/

public class TestMethods extends TestCase {
    private static final Environment EMPTY_ENV = Environment.emptyEnv();

    public TestMethods(String name) {
        super(name);
    }

    public void testCreate() {
        Code m1body = Constructor.returnVal(Constructor.constant(1));
        Code m1 = Constructor.defineMethod(Flags.STATIC, Type.void_type, 
                                           "foo", new ArrayMonoList(0),
                                           new ArrayMonoList(0),
                                           m1body);
        ClassInfo c = new ClassInfo("Foo", "");
        Environment e = new Environment();
        e.setCurrentClass(c);
        Environment env = m1.defineMembers(e);
        MethodInfo mi1 = null;
        try {
            mi1 = env.findMethod(c, "foo", new MethodType(Type.void_type));
        } catch (MethodNotFoundException err) {
            throw new Error("Can't find newly defined method foo");
        }
        MethodInfo mi2 = new MethodInfo(c, Flags.STATIC, "foo", new MethodType(Type.void_type));
        assertEquals("Check method equality", mi1, mi2);
    }

    public void testCodeCreate1() {
        Code m1body = Constructor.constant(1);
        ClosedCode m1code = m1body.eval(EMPTY_ENV);
        Instr code2 = Instr.genConst(1);
        assertEquals("Check body equality", m1code.bytecode, code2);
    }

    public void testCodeCreate2() {
        Code m1body = Constructor.returnVal(Constructor.constant(1));
        ClosedCode m1code = m1body.eval(EMPTY_ENV);
        Instr code2 = Instr.genConst(1).addReturn(Type.int_type);
        assertEquals("Check body equality", m1code.bytecode, code2);
    }

    public void testBodyCreate() {
        Code m1body = Constructor.returnVal(Constructor.constant(1));
        Code m1 = Constructor.defineMethod(Flags.STATIC, Type.int_type,
                                           "foo", new ArrayMonoList(0),
                                           new ArrayMonoList(0),
                                           m1body);
        ClassInfo c = new ClassInfo("Foo", "");
        Environment e = new Environment();
        e.setCurrentClass(c);
        Environment env = m1.defineMembers(e);
        m1.eval(env);
        MethodInfo mi1 = null;
        try {
            mi1 = env.findMethod(c, "foo", new MethodType(Type.void_type));
        } catch (MethodNotFoundException err) {
            throw new Error("Can't find newly defined method foo");
        }
        MethodInfo mi2 = new MethodInfo(c, Flags.STATIC, "foo", new MethodType(Type.int_type));
        Instr code2 = Instr.genConst(1).addReturn(Type.int_type);
        mi2.addCode(code2);
        assertEquals("Check method equality", mi1, mi2);
    }
}

/* End of class TestMethods*/
