/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * Var.java
 *
 *
 */

package uiuc.Jumbo.Compiler;

import uiuc.Jumbo.Jaemus.*;

/**
 * A class that holds information about a method-local variable.
 * A variable is a local that has an index and optionally a constant value.
 * @version      1.0, 20 Oct 2001
 * @author       Lars Clausen
 *
*/

public class Var extends Local {
    final int index;
    final ConstVal constval;

    public Var(String name, int index, Type type, int flags, Local next) {
        super(name, type, flags, next);
        this.index = index;
        this.constval = null;
    }

    public Var(String name, int index, Type type, int flags, Local next,
               ConstVal cv) {
        super(name, type, flags, next);
        this.index = index;
        this.constval = cv;
    }

    public boolean isConstant() {
        return constval != null;
    }
    public ConstVal constantValue() {
        return constval;
    }
}

/* End of class Var*/
