/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * Instr.java
 *
 *
 */

package uiuc.Jumbo.Jaemus;

import java.io.*;
import java.util.*;

import uiuc.Jumbo.Util.*;

public abstract class Instr {
    protected static final int NIL = -1; /* An instruction that disappears */
    protected static final int NOP = 0;
    protected static final int ACONST_NULL = 1;
    protected static final int ICONST_M1 = 2;
    protected static final int ICONST_0 = 3;
    protected static final int ICONST_1 = 4;
    protected static final int ICONST_2 = 5;
    protected static final int ICONST_3 = 6;
    protected static final int ICONST_4 = 7;
    protected static final int ICONST_5 = 8;
    protected static final int LCONST_0 = 9;
    protected static final int LCONST_1 = 10;
    protected static final int FCONST_0 = 11;
    protected static final int FCONST_1 = 12;
    protected static final int FCONST_2 = 13;
    protected static final int DCONST_0 = 14;
    protected static final int DCONST_1 = 15;
    protected static final int BIPUSH = 16;
    protected static final int SIPUSH = 17;
    protected static final int LDC1 = 18;
    protected static final int LDC1_W = 19;
    protected static final int LDC2_W = 20;
    protected static final int ILOAD = 21;
    protected static final int LLOAD = 22;
    protected static final int FLOAD = 23;
    protected static final int DLOAD = 24;
    protected static final int ALOAD = 25;
    protected static final int ILOAD_0 = 26;
    protected static final int ILOAD_1 = 27;
    protected static final int ILOAD_2 = 28;
    protected static final int ILOAD_3 = 29;
    protected static final int LLOAD_0 = 30;
    protected static final int LLOAD_1 = 31;
    protected static final int LLOAD_2 = 32;
    protected static final int LLOAD_3 = 33;
    protected static final int FLOAD_0 = 34;
    protected static final int FLOAD_1 = 35;
    protected static final int FLOAD_2 = 36;
    protected static final int FLOAD_3 = 37;
    protected static final int DLOAD_0 = 38;
    protected static final int DLOAD_1 = 39;
    protected static final int DLOAD_2 = 40;
    protected static final int DLOAD_3 = 41;
    protected static final int ALOAD_0 = 42;
    protected static final int ALOAD_1 = 43;
    protected static final int ALOAD_2 = 44;
    protected static final int ALOAD_3 = 45;
    protected static final int IALOAD = 46;
    protected static final int LALOAD = 47;
    protected static final int FALOAD = 48;
    protected static final int DALOAD = 49;
    protected static final int AALOAD = 50;
    protected static final int BALOAD = 51;
    protected static final int CALOAD = 52;
    protected static final int SALOAD = 53;
    protected static final int ISTORE = 54;
    protected static final int LSTORE = 55;
    protected static final int FSTORE = 56;
    protected static final int DSTORE = 57;
    protected static final int ASTORE = 58;
    protected static final int ISTORE_0 = 59;
    protected static final int ISTORE_1 = 60;
    protected static final int ISTORE_2 = 61;
    protected static final int ISTORE_3 = 62;
    protected static final int LSTORE_0 = 63;
    protected static final int LSTORE_1 = 64;
    protected static final int LSTORE_2 = 65;
    protected static final int LSTORE_3 = 66;
    protected static final int FSTORE_0 = 67;
    protected static final int FSTORE_1 = 68;
    protected static final int FSTORE_2 = 69;
    protected static final int FSTORE_3 = 70;
    protected static final int DSTORE_0 = 71;
    protected static final int DSTORE_1 = 72;
    protected static final int DSTORE_2 = 73;
    protected static final int DSTORE_3 = 74;
    protected static final int ASTORE_0 = 75;
    protected static final int ASTORE_1 = 76;
    protected static final int ASTORE_2 = 77;
    protected static final int ASTORE_3 = 78;
    protected static final int IASTORE = 79;
    protected static final int LASTORE = 80;
    protected static final int FASTORE = 81;
    protected static final int DASTORE = 82;
    protected static final int AASTORE = 83;
    protected static final int BASTORE = 84;
    protected static final int CASTORE = 85;
    protected static final int SASTORE = 86;
    protected static final int POP = 87;
    protected static final int POP2 = 88;
    protected static final int DUP = 89;
    protected static final int DUP_X1 = 90;
    protected static final int DUP_X2 = 91;
    protected static final int DUP2 = 92;
    protected static final int DUP2_X1 = 93;
    protected static final int DUP2_X2 = 94;
    protected static final int SWAP = 95;
    protected static final int IADD = 96;
    protected static final int LADD = 97;
    protected static final int FADD = 98;
    protected static final int DADD = 99;
    protected static final int ISUB = 100;
    protected static final int LSUB = 101;
    protected static final int FSUB = 102;
    protected static final int DSUB = 103;
    protected static final int IMUL = 104;
    protected static final int LMUL = 105;
    protected static final int FMUL = 106;
    protected static final int DMUL = 107;
    protected static final int IDIV = 108;
    protected static final int LDIV = 109;
    protected static final int FDIV = 110;
    protected static final int DDIV = 111;
    protected static final int IREM = 112;
    protected static final int LREM = 113;
    protected static final int FREM = 114;
    protected static final int DREM = 115;
    protected static final int INEG = 116;
    protected static final int LNEG = 117;
    protected static final int FNEG = 118;
    protected static final int DNEG = 119;
    protected static final int ISHL = 120;
    protected static final int LSHL = 121;
    protected static final int ISHR = 122;
    protected static final int LSHR = 123;
    protected static final int IUSHR = 124;
    protected static final int LUSHR = 125;
    protected static final int IAND = 126;
    protected static final int LAND = 127;
    protected static final int IOR = 128;
    protected static final int LOR = 129;
    protected static final int IXOR = 130;
    protected static final int LXOR = 131; 
    protected static final int IINC = 132;
    protected static final int I2L = 133;
    protected static final int I2F = 134;
    protected static final int I2D = 135;
    protected static final int L2I = 136;
    protected static final int L2F = 137;
    protected static final int L2D = 138;
    protected static final int F2I = 139;
    protected static final int F2L = 140;
    protected static final int F2D = 141;
    protected static final int D2I = 142;
    protected static final int D2L = 143;
    protected static final int D2F = 144;
    protected static final int INT2BYTE = 145;
    protected static final int INT2CHAR = 146;
    protected static final int INT2SHORT = 147;
    protected static final int LCMP = 148;
    protected static final int FCMPL = 149; 
    protected static final int FCMPG = 150;
    protected static final int DCMPL = 151;
    protected static final int DCMPG = 152;
    protected static final int IFEQ = 153;
    protected static final int IFNE = 154;
    protected static final int IFLT = 155;
    protected static final int IFGE = 156;
    protected static final int IFGT = 157;
    protected static final int IFLE = 158;
    protected static final int IF_ICMPEQ = 159;
    protected static final int IF_ICMPNE = 160;
    protected static final int IF_ICMPLT = 161;
    protected static final int IF_ICMPGE = 162;
    protected static final int IF_ICMPGT = 163;
    protected static final int IF_ICMPLE = 164;
    protected static final int IF_ACMPEQ = 165;
    protected static final int IF_ACMPNE = 166;
    protected static final int GOTO = 167;
    protected static final int JSR = 168;
    protected static final int RET = 169;
    protected static final int TABLESWITCH = 170;
    protected static final int LOOKUPSWITCH = 171;
    protected static final int IRETURN = 172;
    protected static final int LRETURN = 173;
    protected static final int FRETURN = 174;
    protected static final int DRETURN = 175;
    protected static final int ARETURN = 176;
    protected static final int RETURN = 177;
    protected static final int GETSTATIC = 178;
    protected static final int PUTSTATIC = 179;
    protected static final int GETFIELD = 180;
    protected static final int PUTFIELD = 181;
    protected static final int INVOKEVIRTUAL = 182;
    protected static final int INVOKENONVIRTUAL = 183;
    protected static final int INVOKESTATIC = 184;
    protected static final int INVOKEINTERFACE = 185;
    /*  protected static final int  = 186;*/
    protected static final int NEW = 187;
    protected static final int NEWARRAY = 188;
    protected static final int ANEWARRAY = 189;
    protected static final int ARRAYLENGTH = 190;
    protected static final int ATHROW = 191;
    protected static final int CHECKCAST = 192;
    protected static final int INSTANCEOF = 193;
    protected static final int MONITORENTER = 194;
    protected static final int MONITOREXIT = 195;
    protected static final int WIDE = 196;
    protected static final int MULTIANEWARRAY = 197;
    protected static final int IFNULL = 198;
    protected static final int IFNONNULL = 199;
    protected static final int GOTO_W = 200;
    protected static final int JSR_W = 201;
    protected static final int BREAKPOINT = 202;

    protected static final String[] codenames =
    {
    "nop", "aconst", "iconst_m1", "iconst_0", "iconst_1", "iconst_2", "iconst_3",
    "iconst_4", "iconst_5", "lconst_0", "lconst_1", "fconst_0", "fconst_1",
    "fconst_2", "dconst_0", "dconst_1", "bipush", "sipush", "ldc1", "ldc2",
    "ldc2w", "iload", "lload", "fload", "dload", "aload", "iload_0",
    "iload_1", "iload_2", "iload_3", "lload_0", "lload_1", "lload_2", "lload_3",
    "fload_0", "fload_1", "fload_2", "fload_3", "dload_0", "dload_1", "dload_2",
    "dload_3", "aload_0", "aload_1", "aload_2", "aload_3", "iaload", "laload",
    "faload", "daload", "aaload", "baload", "caload", "saload",
    "istore", "lstore", "fstore", "dstore", "astore", "istore_0",
    "istore_1", "istore_2", "istore_3", "lstore_0", "lstore_1", "lstore_2",
    "lstore_3", "fstore_0", "fstore_1", "fstore_2", "fstore_3", "dstore_0",
    "dstore_1", "dstore_2", "dstore_3", "astore_0", "astore_1", "astore_2",
    "astore_3", "iastore", "lastore", "fastore", "dastore", "aastore",
    "bastore", "castore", "sastore", "pop", "pop2", "dup", "dup",
    "dup", "dup2", "dup2", "dup2", "swap", "iadd", "ladd", "fadd",
    "dadd", "isub", "lsub", "fsub", "dsub", "imul", "lmul", "fmul",
    "dmul", "idiv", "ldiv", "fdiv", "ddiv", "irem", "lrem", "frem",
    "drem", "ineg", "lneg", "fneg", "dneg", "ishl", "lshl", "ishr",
    "lshr", "iushr", "lushr", "iand", "land", "ior", "lor", "ixor",
    "lxor", "iinc", "i2l", "i2f", "i2d", "l2i", "l2f", "l2d", "f2i",
    "f2l", "f2d", "d2i", "d2l", "d2f", "int2byte", "int2char",
    "int2short", "lcmp", "fcmpl", "fcmpg", "dcmpl", "dcmpg", "ifeq",
    "ifne", "iflt", "ifge", "ifgt", "ifle", "if_icmpeq", "if_icmpne",
    "if_icmplt", "if_icmpge", "if_icmpgt", "if_icmple", "if_acmpeq",
    "if_acmpne", "goto", "jsr", "ret", "tableswitch",
    "lookupswitch", "ireturn", "lreturn", "freturn", "dreturn",
    "areturn", "return", "getstatic", "putstatic", "getfield",
    "putfield", "invokevirtual", "invokenonvirtual", "invokestatic",
    "invokeinterface", "missing", "new", "newarray", "anewarray",
    "arraylength", "athrow", "checkcast", "instanceof",
    "monitorenter", "monitorexit", "wide", "multianewarray", "ifnull",
    "ifnonnull", "goto", "jsr", "breakpoint"
    }; 

    public static int[] stack_change =
    {
        0, +1, +1, +1,  /*   0 -   3 */
        +1, +1, +1, +1, /*   4 -   7 */
        +1, +2, +2, +1, /*   8 -  11 */
        +1, +1, +2, +2, /*  12 -  15 */
        +1, +1, +1, +1, /*  16 -  19 */
        +2, +1, +2, +1, /*  20 -  23 */
        +2, +1, +1, +1, /*  24 -  27 */
        +1, +1, +2, +2, /*  28 -  31 */
        +2, +2, +1, +1, /*  32 -  35 */
        +1, +1, +2, +2, /*  36 -  39 */
        +2, +2, +1, +1, /*  40 -  43 */
        +1, +1, -1, +0, /*  44 -  47 */
        -1, +0, -1, -1, /*  48 -  51 */
        -1, -1, -1, -2, /*  52 -  55 */
        -1, -2, -1, -1, /*  56 -  59 */
        -1, -1, -1, -2, /*  60 -  63 */
        -2, -2, -2, -1, /*  64 -  67 */
        -1, -1, -1, -2, /*  68 -  71 */
        -2, -2, -2, -1, /*  72 -  75 */
        -1, -1, -1, -3, /*  76 -  79 */
        -4, -3, -4, -3, /*  80 -  83 */
        -3, -3, -3, -1, /*  84 -  87 */
        -2, +1, +1, +1, /*  88 -  91 */
        +2, +2, +2, +0, /*  92 -  95 */
        -1, -2, -1, -2, /*  96 -  99 */
        -1, -2, -1, -2, /* 100 - 103 */
        -1, -2, -1, -2, /* 104 - 107 */
        -1, -2, -1, -2, /* 108 - 111 */
        -1, -2, -1, -2, /* 112 - 115 */
        -1, -2, -1, -2, /* 116 - 119 */
        -1, -2, -1, -2, /* 120 - 123 */
        -1, -2, -1, -2, /* 124 - 127 */
        -1, -2, -1, -2, /* 128 - 131 */
        +0, +1, +0, +1, /* 132 - 135 */
        -1, -1, +0, +0, /* 136 - 139 */
        +1, +1, -1, +0, /* 140 - 143 */
        -1, +0, +0, +0, /* 144 - 147 */
        -3, -1, -1, -3, /* 148 - 151 */
        -3, -1, -1, -1, /* 152 - 155 */
        -1, -1, -1, -2, /* 156 - 159 */
        -2, -2, -2, -2, /* 160 - 163 */
        -2, -2, -2, +0, /* 164 - 167 */
         0, +0, -1, -1, /* 168 - 171 */ /* JSR */
        -1, -2, -1, -2, /* 172 - 175 */
        -1, +0,  0,  0, /* 176 - 179 */ /* {GET,PUT}STATIC */
         0,  0,  0,  0, /* 180 - 183 */ /*{GET,PUT}FIELD, INVOKE{,NON}VIRTUAL*/
         0,  0,  0, +1, /* 184 - 187 */ /* INVOKE{STATIC,INTERFACE}, RESERVED*/
        +0, +0, +0,  0, /* 188 - 191 */ /* ATHROW */
        -1, +0, -1, +0, /* 192 - 195 */ 
         0,  0, -1, -1, /* 196 - 199 */ /* WIDE, MULTIANEWARRAY */
        +0,  0,  0      /* 200 - 202 */ /* JSR_W, BREAKPOINT */
    };

    public static final int CMP_EQ = 0;
    public static final int CMP_NE = 1;
    public static final int CMP_LT = 2;
    public static final int CMP_GE = 3;
    public static final int CMP_GT = 4;
    public static final int CMP_LE = 5;

    /* Useful arithmetic ops can be had with (op-ADD)*4 */
    public static final int ADD = 6;
    public static final int SUB = 7;
    public static final int MUL = 8;
    public static final int DIV = 9;
    public static final int REM = 10;
    public static final int NEG = 11;

    /* Useful bitwise ops can be had with (op-SHL)*2 */
    public static final int SHL = 12;
    public static final int SHR = 13;
    public static final int USHR = 14;
    public static final int AND = 15;
    public static final int OR = 16;
    public static final int XOR = 17;

    public static final int L = 0;
    public static final int G = 1;

    /** The next instruction in the code sequence 
     */
    Instr next;
    /** The last instruction in the code sequence.  This pointer is only
     *  valid for the first instruction in a sequence.
     */
    Instr last;

    /** Create a new instruction.
     *  @param code The bytecode for this instruction
     */
    Instr(int code) {
        this.code = code;
        last = this;
    }

    /** @return the size of this individual instruction.
     */
    public int size() {
        return 1;
    }

    /** @return the size of the entire chain of instructions. 
     */
    public int totalSize() {
        int total_size = 0;
        for (Instr instr = this; instr != null; instr = instr.next) {
            total_size += instr.size();
        }
        return total_size;
    }

    /** Offset into the whole chunk of bytecode in the method.
     *  Used when code is being written out.
     */
    int offset = -1;
  
    /** Sets the instruction's offset.
     *  @param offset New offset of this instruction.
     */
    protected void setOffset(int offset) {
        this.offset = offset;
        return;
    }

    /** Returns the local variable number used by this instruction */
    protected int getLocalIndex() {
        return -1;
    }

    /** The instruction number (from the VM spec) of this instruction.
     *  Some instructions may be optimized at output-time.
     */
    int code;

    /** Prepares the code for emission.
     * Puts constants in constant pools and assigns offsets.
     */
    public void prepareForEmit(ConstantPool c) {
    }

    /** Emits the instruction as Java bytecode.
     *  @param o The output stream to write bytecode on
     */
    public void emitTo(DataOutputStream o) throws IOException {
        o.writeByte(code);
    }

    /** The list of exception handlers in this list of instructions */
    MonoList handlers;

    /** Append one instruction sequence to another 
     */
    public Instr append(Instr i2) {
        last.next = i2;
        last = i2.last;
     
        if (i2.handlers != null) {
            if (handlers == null) handlers = new ArrayMonoList();
            handlers = handlers.addAll(i2.handlers);
        }
        return this;
    }

    public String toString() {
        return toStringThis()+(next!=null?","+next.toString():"");
    }

    public String toStringThis() {
        return codenames[code];
    }

    public void prettyPrint() {
        for (Instr instr = this; instr != null; instr = instr.next) {
            System.out.println(instr.offset+": "+instr.toStringThis());
        }
    }

    /*************************************************************/
    /* Bytecode-generating instructions begin here               */
    /*************************************************************/

    /* For backwards compatability */
    public static Instr genIconst(int v) {
        return genConst(v);
    }

    public final Instr addIconst(int v) {
        return append(genConst(v));
    }

    public boolean equals(Object other) {
        if (!(other instanceof Instr)) return false;
        Instr i = (Instr)other;
        if (i == null) return false;
        if (code != i.code) return false;
        if (handlers != null) {
            if (!handlers.equals(i.handlers))
                return false;
        } else {
            if (i.handlers != null) return false;
        }
        // Skip over Nil instructions, as they have no meaning
        Instr this1 = this;
        while ((this1.next != null) && (this1.next instanceof NilArg))
            this1 = this1.next;
        while ((i.next != null) && (i.next instanceof NilArg))
            i = i.next;
        if (this1.next != null) {
            if (i.next == null) return false;
            return this1.next.equals(i.next);
        }
        else
            return i.next == null;
    }

    /** Create an instruction that pushes an int constant onto the stack.
     *  @param i The value of the constant
     */
    public static final Instr genConst(int i) {
        if (i > -2 && i < 6) 
            return new NoArg(ICONST_M1+(i+1));
        else {
            if ((i >= Byte.MIN_VALUE) && (i <= Byte.MAX_VALUE)) {
                return new ByteArg(BIPUSH, i);
            }
            else if ((i >= Short.MIN_VALUE) && (i <= Short.MAX_VALUE)) {
                return new ShortArg(SIPUSH, i);
            }
            else {
                Constant c = new ConstantInteger(i);

                return new ConstantArg(LDC1_W, c);
            }
        }
    }
    /** Create an instruction that pushes a float constant onto the stack.
     *  @param f The value of the constant
     */
    public static final Instr genConst(float f) {
        if (f == 0.0) return new NoArg(FCONST_0);
        if (f == 1.0) return new NoArg(FCONST_1);
        if (f == 2.0) return new NoArg(FCONST_2);
        Constant c = new ConstantFloat(f);
        return new ConstantArg(LDC1_W, c);
    }
    /** Create an instruction that pushes a double constant onto the stack.
     *  @param d The value of the constant
     */
    public static final Instr genConst(double d) {
        if (d == 0.0) return new NoArg(DCONST_0);
        if (d == 1.0) return new NoArg(DCONST_1);
        Constant c = new ConstantDouble(d);
        return new ConstantArg(LDC2_W, c);
    }
    /** Create an instruction that pushes a long constant onto the stack.
     *  @param l The value of the constant
     */
    public static final Instr genConst(long l) {
        if (l == 0) return new NoArg(LCONST_0);
        if (l == 1) return new NoArg(LCONST_1);
        Constant c = new ConstantLong(l);
        return new ConstantArg(LDC2_W, c);
    }
    public static final Instr genConst(int i, Type type) {
        if (type == Type.long_type) return genConst((long)i);
        if (type == Type.float_type) return genConst((float)i);
        if (type == Type.double_type) return genConst((double)i);
        return genConst(i);
    }
    public static final Instr genConst(String s) {
        Constant c = new ConstantString(s);
        return new ConstantArg(LDC1_W, c);
    }
    public static final Instr genConst(boolean b) {
        if (b) return new NoArg(ICONST_1);
        else return new NoArg(ICONST_0);
    }
    public final Instr addConst(int i) { return append(genIconst(i)); }
    public final Instr addConst(float f) { return append(genConst(f)); }
    public final Instr addConst(double d) { return append(genConst(d)); }
    public final Instr addConst(long l) { return append(genConst(l)); }
    public final Instr addConst(int i, Type type) {
        return append(genConst(i, type));
    }
    public final Instr addConst(String s) { return append(genConst(s)); }
    public final Instr addConst(boolean b) { return append(genConst(b)); }

    public static final Instr genNull() { return new NoArg(ACONST_NULL); }
    public final Instr addNull() { return append(genNull()); }

    /** Create a 'load' instruction
     *  @param i The index of the variable to load from
     *  @param t The type of the variable being loaded
     */
    public static final Instr genLoad(int i, Type t) {
        switch (t.kind) {
        case Type.BYTE:
        case Type.CHAR:
        case Type.SHORT:
        case Type.INT:
        case Type.BOOLEAN:
            if (i < 4) return new NoArg(ILOAD_0+i);
            else return new ByteArg(ILOAD, i);
        case Type.LONG:
            if (i < 4) return new NoArg(LLOAD_0+i);
            else return new ByteArg(LLOAD, i);
        case Type.FLOAT:
            if (i < 4) return new NoArg(FLOAD_0+i);
            else return new ByteArg(FLOAD, i);
        case Type.DOUBLE:
            if (i < 4) return new NoArg(DLOAD_0+i);
            else return new ByteArg(DLOAD, i);
        case Type.OBJECT:
        case Type.ARRAY:
        case Type.NULL:
            if (i < 4) return new NoArg(ALOAD_0+i);
            else return new ByteArg(ALOAD, i);
        case Type.VOID:
            throw new Error("Can't load void values");
        case Type.CLASS:
            throw new Error("Can't load class values");
        default:
            throw new Error("Unknown kind "+t.kind+" for load "+i);
        }
    }
    public final Instr addLoad(int i, Type t) {
        return append(genLoad(i, t));
    }

    /** Create an array load instruction.
     * @param t type of array load instruction
     */
    public static final Instr genALoad(Type t) {
        switch(t.kind) {
        case Type.INT:
            return new NoArg(IALOAD);
        case Type.LONG:
            return new NoArg(LALOAD);
        case Type.FLOAT:
            return new NoArg(FALOAD);
        case Type.DOUBLE:
            return new NoArg(DALOAD);
        case Type.OBJECT:
        case Type.ARRAY:
            return new NoArg(AALOAD);
        case Type.BYTE:
        case Type.BOOLEAN:
            return new NoArg(BALOAD);
        case Type.CHAR:
            return new NoArg(CALOAD);
        case Type.SHORT:
            return new NoArg(SALOAD);
        case Type.VOID:
            throw new Error("Can't load void values");
        case Type.CLASS:
            throw new Error("Can't load class values");
        case Type.NULL:
            throw new Error("Can't have an array of NULL");
        default:
            throw new Error("Unknown kind "+t.kind+" for aload");
        }
    }
    public final Instr addALoad(Type t) {
        return append(genALoad(t));
    }

    /** Create an array store instruction.
     * @param t type of array store instruction
     */
    public static final Instr genAStore(Type t) {
        switch(t.kind) {
        case Type.INT:
            return new NoArg(IASTORE);
        case Type.LONG:
            return new NoArg(LASTORE);
        case Type.FLOAT:
            return new NoArg(FASTORE);
        case Type.DOUBLE:
            return new NoArg(DASTORE);
        case Type.OBJECT:
        case Type.ARRAY:
            return new NoArg(AASTORE);
        case Type.BYTE:
        case Type.BOOLEAN:
            return new NoArg(BASTORE);
        case Type.CHAR:
            return new NoArg(CASTORE);
        case Type.SHORT:
            return new NoArg(SASTORE);
        case Type.VOID:
            throw new Error("Can't have an array of VOID");
        case Type.CLASS:
            throw new Error("Can't have an array of CLASS");
        case Type.NULL:
            throw new Error("Can't have an array of NULL");
        default:
            throw new Error("Unknown kind "+t.kind+" for aload");
        }
    }
    public final Instr addAStore(Type t) {
        return append(genAStore(t));
    }

    /** Create a type conversion instruction.
     * @param from  original type is converted to...
     * @param to    ... new type
     */
    public static final Instr genConvert(Type from, Type to) {
        if (from.kind == to.kind)
            return new NilArg("No conversion necessary");
        switch(from.kind) {
        case Type.INT:
        case Type.SHORT:
        case Type.BYTE:
        case Type.CHAR:
            switch(to.kind) {
            case Type.INT: return new NilArg("No integer conversion necessary");
            case Type.LONG: return new NoArg(I2L);
            case Type.FLOAT: return new NoArg(I2F);
            case Type.DOUBLE: return new NoArg(I2D);
            case Type.BYTE: return new NoArg(INT2BYTE);
            case Type.CHAR: return new NoArg(INT2CHAR);
            case Type.SHORT: return new NoArg(INT2SHORT);
            default: throw new Error("Can't convert " + from + " to " + to);
            }
        case Type.LONG:
            switch(to.kind) {
            case Type.INT: return new NoArg(L2I);
            case Type.BYTE: 
            case Type.SHORT:
            case Type.CHAR:
                return new NoArg(L2I).addConvert(Type.int_type, to);
            case Type.FLOAT: return new NoArg(L2F);
            case Type.DOUBLE: return new NoArg(L2D);
            default: throw new Error("Can't convert " + from + " to " + to);
            }
        case Type.FLOAT:
            switch(to.kind) {
            case Type.INT: return new NoArg(F2I);
            case Type.BYTE: 
            case Type.SHORT:
            case Type.CHAR:
                return new NoArg(F2I).addConvert(Type.int_type, to);
            case Type.LONG: return new NoArg(F2L);
            case Type.DOUBLE: return new NoArg(F2D);
            default: throw new Error("Can't convert " + from + " to " + to);
            }
        case Type.DOUBLE:
            switch(to.kind) {
            case Type.INT: return new NoArg(D2I);
            case Type.BYTE: 
            case Type.SHORT:
            case Type.CHAR:
                return new NoArg(D2I).addConvert(Type.int_type, to);
            case Type.LONG: return new NoArg(D2L);
            case Type.FLOAT: return new NoArg(D2F);
            default: throw new Error("Can't convert " + from + " to " + to);
            }
        default:
            throw new Error("Can't convert " + from + " to "+to);
        }
    }
    public final Instr addConvert(Type from, Type to) {
        return append(genConvert(from, to));
    }

    public static final Instr genArith(int o, Type t) {
        if (o >= SHL) return genBitwise(o, t);
        else return genMath(o, t);
    }

    public final Instr addArith(int o, Type t) {
        return append(genArith(o, t));
    }

    /** Create a bitwise op instruction.
     * @param o   type of logcial operation (SHL, OR, etc)
     * @param t   type of the operand(s)
     */
    public static final Instr genBitwise(int o, Type t) {
        switch(t.kind) {
        case Type.BYTE:
        case Type.CHAR:
        case Type.SHORT:
        case Type.INT:
            return new NoArg(ISHL + (o-SHL)*2);
        case Type.BOOLEAN:
            if (o >= AND)
                return new NoArg(ISHL + (o-SHL)*2);
            else
                throw new Error("Can't do shift ("+o+") on booleans");
        case Type.LONG:
            return new NoArg(LSHL + (o-SHL)*2);
        case Type.FLOAT:
        case Type.DOUBLE:
        case Type.OBJECT:
        case Type.ARRAY:
        case Type.VOID:
        case Type.CLASS:
        case Type.NULL:
            throw new Error("Can't do bitwise ("+o+") on objects of type "+t);
        default:
            throw new Error("Unknown kind "+t.kind+" for bitwise ("+o+")");
        }
    }
    public final Instr addBitwise(int o, Type t) {
        return append(genBitwise(o, t));
    }

    /** Create an arithmetic instruction.
     * @param o   type of arithmetic operation (ADD, SUB, NEG, etc)
     * @param t   type of the operand(s)
     */
    public static final Instr genMath(int o, Type t) {
        switch(t.kind) {
        case Type.BYTE:
        case Type.CHAR:
        case Type.SHORT:
        case Type.INT:
            return new NoArg(IADD + (o-ADD)*4);
        case Type.LONG:
            return new NoArg(LADD + (o-ADD)*4);
        case Type.FLOAT:
            return new NoArg(FADD + (o-ADD)*4);
        case Type.DOUBLE:
            return new NoArg(DADD + (o-ADD)*4);
        case Type.OBJECT:
        case Type.ARRAY:
        case Type.VOID:
        case Type.CLASS:
        case Type.NULL:
        case Type.BOOLEAN:
            throw new Error("Can't do math ("+o+") on objects of type "+t);
        default:
            throw new Error("Unknown kind "+t.kind+" for math ("+o+")");
        }
    }
    public final Instr addMath(int o, Type t) {
        return append(genMath(o, t));
    }

    /** Create an 'add' instruction.
     *  @param t  type of addition instruction
     */
    public static final Instr genAdd(Type t) {
        return genMath(ADD, t);
    }
    public final Instr addAdd(Type t) {
        return append(genMath(ADD, t));
    }

    /** Creates a 'store' instruction.
     * @param i Index into the local variable stack
     * @param t Type of variable being stored into
     */
    public static final Instr genStore(int index, Type t) {
        // true if we can use a shortcut instruction like ISTORE_0.
        boolean shortcut = (index >= 0) && (index <= 3);

        switch(t.kind) {
        case Type.BYTE:
        case Type.CHAR:
        case Type.SHORT:
        case Type.INT:
        case Type.BOOLEAN:
            if (shortcut) return new NoArg(ISTORE_0 + index);
            else          return new ByteArg(ISTORE, index);
        case Type.LONG:
            if (shortcut) return new NoArg(LSTORE_0 + index);
            else          return new ByteArg(LSTORE, index);
        case Type.FLOAT:
            if (shortcut) return new NoArg(FSTORE_0 + index);
            else          return new ByteArg(FSTORE, index);
        case Type.DOUBLE:
            if (shortcut) return new NoArg(DSTORE_0 + index);
            else          return new ByteArg(DSTORE, index);
        case Type.OBJECT:
        case Type.ARRAY:
        case Type.NULL:
            if (shortcut) return new NoArg(ASTORE_0 + index);
            else          return new ByteArg(ASTORE, index);
        case Type.CLASS:
            throw new Error("Store into a class type?");
        case Type.VOID:
            throw new Error("Store into a void type?");
        default:
            throw new Error("Unknown kind "+t.kind+" for store");
        }
    }
    public final Instr addStore(int index, Type t) {
        return append(genStore(index, t));
    }

    /** Creates a Jump Subroutine instruction.
     * @param target The target of the subroutine jump.
     */
    public static final Instr genJsr(Instr target) {
        return new LabelArg(JSR, target);
    }
    public final Instr addJsr(Instr target) { return append(genJsr(target)); }

    /** Creates a ret(urn) (from subroutine) instruction
     * @param addressvar The local variable holding the return address.
     * Note that Jsr and Ret are asymmetric: Jsr jumps to a given target,
     * then leaves the return address on the stack.  Ret reads the return
     * address from a local variable.  The return address is then expected
     * to have been put there at some point in between.
     */
    public static final Instr genRet(int addressvar) {
        return new ByteArg(RET, addressvar);
    }
    public final Instr addRet(int addressvar) {
        return append(genRet(addressvar));
    }

    /** Creates a return (from method) instruction.
     * @param t Type of the return argument.
     */
    public static final Instr genReturn(Type t) {
        switch (t.kind) {
        case Type.BOOLEAN:
        case Type.BYTE:
        case Type.CHAR:
        case Type.SHORT:
        case Type.INT:
            return new NoArg(IRETURN);
        case Type.LONG:
            return new NoArg(LRETURN);
        case Type.FLOAT:
            return new NoArg(FRETURN);
        case Type.DOUBLE:
            return new NoArg(DRETURN);
        case Type.OBJECT:
        case Type.ARRAY:
        case Type.NULL:
            return new NoArg(ARETURN);
        case Type.VOID:
        case Type.CLASS:
            return new NoArg(RETURN);
        default:
            throw new Error("Unknown kind "+t.kind+" for return");
        }
    }

    public final Instr addReturn(Type t) {
        return append(genReturn(t));
    }

    /** Creates a pop instruction.
     * @param t Operand type to be popped.
     */
    public static final Instr genPop(Type t) {
        switch (t.kind) {
        case Type.BOOLEAN:
        case Type.BYTE:
        case Type.CHAR:
        case Type.SHORT:
        case Type.INT:
        case Type.FLOAT:
        case Type.OBJECT:
        case Type.ARRAY:
        case Type.NULL:
            return new NoArg(POP);
        case Type.LONG:
        case Type.DOUBLE:
            return new NoArg(POP2);
        case Type.VOID:
        case Type.CLASS:
        case Type.BOTTOM:
            return new NilArg("Popping nothing");
        default:
            throw new Error("Unknown kind "+t.kind+" for pop");
        }
    }
    public final Instr addPop(Type t) {
        return append(genPop(t));
    }

    /** Duplicates one or two items on the stack, optionally pushing them down.
     * @param dup_items The number of stack items to duplicate.
     * @param pushdown The number of stack items to put the duplicate under.
     * Note that a Long or Double counts as two stack items.
     */
    public static final Instr genDup(int dup_items, int pushdown) {
        if (dup_items < 1 || dup_items > 2) {
            throw new Error("Can only dup one or two items, not "+dup_items);
        }
        if (pushdown < 0 || pushdown > 2) {
            throw new Error("Can only push down zero to two places, not "+pushdown);
        }
        return new NoArg(DUP+(dup_items-1)*3+pushdown);
    }
    public final Instr addDup(int dup_items, int pushdown) {
        return append(genDup(dup_items, pushdown));
    }
    public static final Instr genDup(Type itemtype, int pushdown) {
        return genDup(itemtype.wordSize(), pushdown);
    }
    public final Instr addDup(Type itemtype, int pushdown) {
        return append(genDup(itemtype.wordSize(), pushdown));
    }
    public static final Instr genDup(Type itemtype, Type pushdown) {
        return genDup(itemtype.wordSize(), pushdown.wordSize());
    }
    public final Instr addDup(Type itemtype, Type pushdown) {
        return append(genDup(itemtype.wordSize(), pushdown.wordSize()));
    }
    public static final Instr genDup(Type itemtype) {
        return genDup(itemtype, Type.void_type);
    }
    public final Instr addDup(Type itemtype) {
        return addDup(itemtype, Type.void_type);
    }
  
    /** Creates an "if" (compare) instruction that compares to 0.
     * @param compare_kind Kind of comparision (CMP_LE, etc)
     * @param target Instruction to jump to if the comparision is true
     */
    public static final Instr genIf(int compare_kind, Instr target) {
        if (compare_kind < 0 || compare_kind > CMP_LE) {
            throw new Error("Unknown compare kind "+compare_kind+" for if");
        }
        return new LabelArg(IFEQ+compare_kind, target);
    }
    public final Instr addIf(int compare_kind, Instr target) {
        return append(genIf(compare_kind, target));
    }

    /** Creates an "if" instruction that compares integers.
     * @param compare_kind Kind of comparision (CMP_LE, etc)
     * @param target Instruction to jump to if the comparision is true
     */
    public static final Instr genIfIcomp(int compare_kind, Instr target) {
        if (compare_kind < 0 || compare_kind > CMP_LE) {
            throw new Error("Unknown compare kind "+compare_kind+" for if_icomp");
        }
        return new LabelArg(IF_ICMPEQ+compare_kind, target);
    }
    public final Instr addIfIcomp(int compare_kind, Instr target) {
        return append(genIfIcomp(compare_kind, target));
    }

    /** Creates an "if" instruction that compares the addresses of two objects.
     * @param compare_kind Kind of comparision (limited to equality/non-equality).
     * @param target Instruction to jump to if the comparision is true.
     */
    public static final Instr genIfAcomp(int compare_kind, Instr target) {
        if (compare_kind < 0 || compare_kind > CMP_NE) {
            throw new Error("Unknown  or illegal compare kind "+compare_kind+" for if_acomp");
        }
        return new LabelArg(IF_ACMPEQ+compare_kind, target);
    }
    public final Instr addIfAcomp(int compare_kind, Instr target) {
        return append(genIfAcomp(compare_kind, target));
    }

    /** Creates an "if" instruction that checks for 'null'.
     * @param compare_kind Kind of comparision (limited to equality/non-equality).
     * @param target Instruction to jump to if the comparision is true.
     */
    public static final Instr genIfNull(int compare_kind, Instr target) {
        if (compare_kind < 0 || compare_kind > CMP_NE) {
            throw new Error("Unknown compare kind "+compare_kind+" for ifnull");
        }
        return new LabelArg(IFNULL+compare_kind, target);
    }
    public final Instr addIfNull(int compare_kind, Instr target) {
        return append(genIfNull(compare_kind, target));
    }

    /** Creates a goto instruction.
     * @param target Instruction to jump to.
     */
    public static final Instr genGoto(Instr target) {
        return new LabelArg(GOTO, target);
    }
    public final Instr addGoto(Instr target) {
        return append(genGoto(target));
    }

    /** Creates a swap instruction.
     */
    public static final Instr genSwap() {
        return new NoArg(SWAP);
    }
    public final Instr addSwap() {
        return append(genSwap());
    }
    
    /** Creates a dummy 'nil' instruction. */
    public static final Instr genNil() { return new NilArg(""); }
    public final Instr addNil() { return append(genNil()); }

    /** Creates a dummy 'nil' comment instruction. */
    public static final Instr genNil(String comment) { return new NilArg(comment); }
    public final Instr addNil(String comment) { return append(genNil(comment)); }

    /** Creates an i(nteger)inc(rement) instruction.
     * @param localvar local variable index
     * @param amount amount to increment by.
     */
    public static final Instr genIinc(int localvar, int amount) {
        if (localvar < 0 || localvar > 255) {
            // XXX Fixme: Could use Wide here
            throw new Error("Local variable out of range: "+localvar+" in Iinc");
        }
        if (amount > 127 || amount < -128) {
            throw new Error("Iinc amount out of range: "+amount);
        }
        if (amount < 0)
            return new ShortArg(IINC, (localvar<<8)+(256+amount));
        else
            return new ShortArg(IINC, (localvar<<8)+amount);
    }
    public final Instr addIinc(int localvar, int amount) {
        return append(genIinc(localvar, amount));
    }

    /** Creates a new 'new' instruction.
     * @param object class to create a new instance of, from the constant pool
     */
    public static final Instr genNew(Constant object) {
        return new ConstantArg(NEW, object);
    }
    /** Creates a new 'new' instruction.
     * @param cl information about the class to create a new instance of.
     */
    public static final Instr genNew(ClassInfo cl) {
        return genNew(new ConstantClass(cl));
    }
    /** Creates a new 'new' instruction.
     * @param objectname name of the class, as a string, to create a new instance of.
     */
    public static final Instr genNew(String objectname) {
        return genNew(new ConstantClass(objectname));
    }
    public final Instr addNew(Constant object) { return append(genNew(object)); }
    public final Instr addNew(ClassInfo cl) { return append(genNew(cl)); }
    public final Instr addNew(String objectname) { return append(genNew(objectname)); }

    /** Creates an array-creation instruction.
     * @param type type of the array elements
     * @param dims dimensions of the array
     */
    public static final Instr genNewarray(Type type, int dims) {
        if (dims == 1) {
            if (type.kind == Type.OBJECT)
                return new ConstantArg(ANEWARRAY, new ConstantClass(type.className()));
            else if ( type.kind == Type.ARRAY)
                return new ConstantArg(ANEWARRAY, new ConstantClass(type.bytecodeString()));
            else 
                return new ByteArg(NEWARRAY, type.kind);

        } else {
            int dims_count = dims;
            while (dims_count > 0) {
                type = new Type(type);
                dims_count--;
            }
            return new ConstantByteArg(MULTIANEWARRAY, 
                                       new ConstantClass(type.bytecodeString()), dims);
        }
    }
    public final Instr addNewarray(Type type, int dims) {
        return append(genNewarray(type, dims));
    }

    /** Creates an 'arraylength' instruction.
     */
    public static final Instr genArraylength() { return new NoArg(ARRAYLENGTH); }
    public final Instr addArrayLength() { return append(genArraylength()); }
    public static final Instr genThrow() { return new NoArg(ATHROW); }
    public final Instr addThrow() { return append(genThrow()); }

    /** Creates a 'cmp' instruction.
     * @param type type of operands to compare
     */
    public static final Instr genCmp(Type type) {
        switch(type.kind) {
        case Type.LONG:
            return new NoArg(LCMP);
        case Type.FLOAT:
        case Type.DOUBLE:
            throw new Error("Must select either G or L for "+type+" cmp");
        default:
            throw new Error("Unknown type "+type+" for cmp");
        }
    }
    public final Instr addCmp(Type type) { 
        return append(genCmp(type));
    }
    /** Creates a 'cmp' instruction.
     * @param type type of operands to compare
     * @param kind L or G (for float and double cmp)
     */
    public static final Instr genCmp(Type type, int kind) {
        switch(type.kind) {
        case Type.FLOAT:
            return new NoArg(FCMPL + kind);
        case Type.DOUBLE:
            return new NoArg(DCMPL + kind);
        case Type.LONG:
            return new NoArg(LCMP);
        default:
            throw new Error("Unknown type "+type+" for cmp");
        }
    }
    public final Instr addCmp(Type type, int kind) { 
        return append(genCmp(type, kind));
    }

    public static final Instr genCheckcast(Constant exc) {
        return new ConstantArg(CHECKCAST, exc);
    }
    public static final Instr genCheckcast(String excname) {
        return genCheckcast(new ConstantClass(excname));
    }
    public static final Instr genCheckcast(ClassInfo exclass) {
        return genCheckcast(new ConstantClass(exclass.name));
    }
    public final Instr addCheckcast(Constant exc) { return append(genCheckcast(exc)); }
    public final Instr addCheckcast(String excname) { return append(genCheckcast(excname)); }
    public final Instr addCheckcast(ClassInfo exclass) { return append(genCheckcast(exclass)); }

    public static final Instr genInstanceof(Constant exc) {
        return new ConstantArg(INSTANCEOF, exc);
    }
    public static final Instr genInstanceof(String excname) {
        return genInstanceof(new ConstantClass(excname));
    }
    public static final Instr genInstanceof(ClassInfo exclass) {
        return genInstanceof(new ConstantClass(exclass.name));
    }
    public static final Instr genInstanceof(Type t) {
        return genInstanceof(new ConstantClass(t));
    }
    public final Instr addInstanceof(Constant exc) { return append(genInstanceof(exc)); }
    public final Instr addInstanceof(String excname) { return append(genInstanceof(excname)); }
    public final Instr addInstanceof(ClassInfo exclass) { return append(genInstanceof(exclass)); }
    public final Instr addInstanceof(Type t) { return append(genInstanceof(t)); }

    public static final Instr genMonitorenter() { return new NoArg(MONITORENTER); }
    public final Instr addMonitorenter() { return append(genMonitorenter()); }
    public static final Instr genMonitorexit() { return new NoArg(MONITOREXIT); }
    public final Instr addMonitorexit() { return append(genMonitorexit()); }

    /** Creates a new switch instruction.
     * @param cases A map from integers to instructions, which may contain
     * holes.
     * @param default The instruction to jump to when no case matches.
     */
    public static final Instr genSwitch(SortedMap cases, Instr defaultcase) {
        if (cases.size() == 0) {
            return genGoto(defaultcase);
        }

        int first = ((Integer)(cases.firstKey())).intValue();
        int last = ((Integer)(cases.lastKey())).intValue();

        // This is a fairly arbitrary constant.  This is the
        // point where a Lookupswitch takes up less space than a 
        // Tableswitch.
        if ((last-first)/cases.size() < 2) {
            // Fairly dense, use a tableswitch
            MonoList targets = new ArrayMonoList();
            Iterator iter = cases.entrySet().iterator();
            for (int i = first; i <= last; i++) {
                Map.Entry entry = (Map.Entry)iter.next();
                int k = ((Integer)entry.getKey()).intValue();
                while (i < k) {
                    targets.add(defaultcase);
                    i++;
                }
                targets.add(entry.getValue());
            }
            return new TableSwitch(defaultcase, first, last, targets);
        } else {
            return new LookupSwitch(defaultcase, cases);
        }
    }
    public final Instr addSwitch(SortedMap cases, Instr defaultcase) {
        return append(genSwitch(cases, defaultcase));
    }
    
    /* ************************************************************
     * Field and method manipulation instructions
     * ************************************************************ */
    public static final Instr genGetfield(Constant fc) {
        return new ConstantArg(GETFIELD, fc);
    }
    public static final Instr genGetstatic(Constant fc) {
        return new ConstantArg(GETSTATIC, fc);
    }
    public static final Instr genPutfield(Constant fc) {
        return new ConstantArg(PUTFIELD, fc);
    }
    public static final Instr genPutstatic(Constant fc) {
        return new ConstantArg(PUTSTATIC, fc);
    }
    public final Instr addGetfield(Constant fc) { return append(genGetfield(fc)); }
    public final Instr addGetstatic(Constant fc) { return append(genGetstatic(fc)); }
    public final Instr addPutfield(Constant fc) { return append(genPutfield(fc)); }
    public final Instr addPutstatic(Constant fc) { return append(genPutstatic(fc)); }

    public static final Instr genGetfield(FieldInfo fc) {
        return new ConstantArg(GETFIELD, new ConstantFieldRef(fc));
    }
    public static final Instr genGetstatic(FieldInfo fc) {
        return new ConstantArg(GETSTATIC, new ConstantFieldRef(fc));
    }
    public static final Instr genPutfield(FieldInfo fc) {
        return new ConstantArg(PUTFIELD, new ConstantFieldRef(fc));
    }
    public static final Instr genPutstatic(FieldInfo fc) {
        return new ConstantArg(PUTSTATIC, new ConstantFieldRef(fc));
    }
    public final Instr addGetfield(FieldInfo fc) { return append(genGetfield(fc)); }
    public final Instr addGetstatic(FieldInfo fc) { return append(genGetstatic(fc)); }
    public final Instr addPutfield(FieldInfo fc) { return append(genPutfield(fc)); }
    public final Instr addPutstatic(FieldInfo fc) { return append(genPutstatic(fc)); }

    public static final Instr genGetfield(String cl, String name, Type type) {
        return new ConstantArg(GETFIELD, new ConstantFieldRef(cl, name, type.bytecodeString()));
    }
    public static final Instr genGetstatic(String cl, String name, Type type) {
        return new ConstantArg(GETSTATIC, new ConstantFieldRef(cl, name, type.bytecodeString()));
    }
    public static final Instr genPutfield(String cl, String name, Type type) {
        return new ConstantArg(PUTFIELD, new ConstantFieldRef(cl, name, type.bytecodeString()));
    }
    public static final Instr genPutstatic(String cl, String name, Type type) {
        return new ConstantArg(PUTSTATIC, new ConstantFieldRef(cl, name, type.bytecodeString()));
    }
    public final Instr addGetfield(String cl, String name, Type type) { return append(genGetfield(cl, name, type)); }
    public final Instr addGetstatic(String cl, String name, Type type) { return append(genGetstatic(cl, name, type)); }
    public final Instr addPutfield(String cl, String name, Type type) { return append(genPutfield(cl, name, type)); }
    public final Instr addPutstatic(String cl, String name, Type type) { return append(genPutstatic(cl, name, type)); }

    public static final Instr genInvokevirtual(Constant mc) {
        return new ConstantArg(INVOKEVIRTUAL, mc);
    }
    public static final Instr genInvokestatic(Constant mc) {
        return new ConstantArg(INVOKESTATIC, mc);
    }
    public static final Instr genInvokenonvirtual(Constant mc) {
        return new ConstantArg(INVOKENONVIRTUAL, mc);
    }
    public static final Instr genInvokeinterface(Constant mc) {
        return new InterfaceInstr(mc);
    }
    public final Instr addInvokevirtual(Constant mc) { 
        return append(genInvokevirtual(mc)); 
    }
    public final Instr addInvokestatic(Constant mc) { 
        return append(genInvokestatic(mc)); 
    }
    public final Instr addInvokenonvirtual(Constant mc) { 
        return append(genInvokenonvirtual(mc)); 
    }
    public final Instr addInvokeinterface(Constant mc) {
        return append(genInvokeinterface(mc));
    }

    public static final Instr genInvokevirtual(MethodInfo mc) {
        return new ConstantArg(INVOKEVIRTUAL, new ConstantMethodRef(mc));
    }
    public static final Instr genInvokestatic(MethodInfo mc) {
        return new ConstantArg(INVOKESTATIC, new ConstantMethodRef(mc));
    }
    public static final Instr genInvokenonvirtual(MethodInfo mc) {
        return new ConstantArg(INVOKENONVIRTUAL, new ConstantMethodRef(mc));
    }
    public static final Instr genInvokeinterface(MethodInfo mc) {
        return new InterfaceInstr(new ConstantInterfaceMethodRef(mc));
    }
    public final Instr addInvokevirtual(MethodInfo mc) { 
        return append(genInvokevirtual(mc)); 
    }
    public final Instr addInvokestatic(MethodInfo mc) { 
        return append(genInvokestatic(mc)); 
    }
    public final Instr addInvokenonvirtual(MethodInfo mc) { 
        return append(genInvokenonvirtual(mc)); 
    }
    public final Instr addInvokeinterface(MethodInfo mc) {
        return append(genInvokeinterface(mc));
    }

    public static final Instr genInvokevirtual(String cl, String name, MethodType type) {
        return new ConstantArg(INVOKEVIRTUAL, new ConstantMethodRef(cl, name, type));
    }
    public static final Instr genInvokestatic(String cl, String name, MethodType type) {
        return new ConstantArg(INVOKESTATIC, new ConstantMethodRef(cl, name, type));
    }
    public static final Instr genInvokenonvirtual(String cl, String name, MethodType type) {
        return new ConstantArg(INVOKENONVIRTUAL, new ConstantMethodRef(cl, name, type));
    }
    public static final Instr genInvokeinterface(String cl, String name, MethodType type) {
        return new InterfaceInstr(new ConstantMethodRef(cl, name, type));
    }
    public final Instr addInvokevirtual(String cl, String name, MethodType type) { 
        return append(genInvokevirtual(cl, name, type)); 
    }
    public final Instr addInvokestatic(String cl, String name, MethodType type) { 
        return append(genInvokestatic(cl, name, type)); 
    }
    public final Instr addInvokenonvirtual(String cl, String name, MethodType type) { 
        return append(genInvokenonvirtual(cl, name, type)); 
    }
    public final Instr addInvokeinterface(String cl, String name, MethodType type) {
        return append(genInvokeinterface(cl, name, type));
    }

    /** Add an exception handler to a list of instructions.
     * @param start The instruction list handled by the exception handler
     * @param handler The instruction list used as a handler
     * @param type The type of exceptions handled
     */
    public static Instr addHandler(Instr start, Instr handler, Type type) {
        return start.addHandler(handler,type);
    }

    public final Instr addHandler(Instr handler, Type type) {
        if (handlers == null) handlers = new ArrayMonoList();
        handlers.add(new ExceptionInfo(this, last, handler, type.class_name));
        return this;
    }

}

/** Concrete class for the "Nil" dummy instruction (not output)
 **/
class NilArg extends Instr {
    String comment;
    NilArg(String comment) {
        super(NIL);
        this.comment = comment;
    }
    public void emitTo(DataOutputStream o) {
        /* Nothing is emitted */
    }
    public int size() { return 0; }
    public String toStringThis() {
        return "\""+comment+"\"";
    }
}

/** Concrete class for instructions that take no arguments.
 **/
class NoArg extends Instr {
    NoArg(int code) {
        super(code);
    }
    protected int getLocalIndex() {
        if (code >= ILOAD_0 && code <= ALOAD_3) {
            int x = code-ILOAD_0;
            return x%4+(x&0x4)/4;
        }
        if (code >= ISTORE_0 && code <= ASTORE_3) {
            int x = code-ISTORE_0;
            return x%4+(x&0x4)/4;
        }
        return -1;
    }
}

/** Concrete class for instructions that take a byte argument.
 **/
class ByteArg extends Instr {
    int arg;
    ByteArg(int code, int arg) {
        super(code);
        if (arg > 255) throw new Error("Wide instruction not yet implemented");
        this.arg = arg;
    }
    protected int getLocalIndex() {
        if (code >= ILOAD && code <= ALOAD) {
            int x = code-ILOAD;
            return arg+(x&1);
        }
        if (code >= ISTORE && code <= ASTORE) {
            int x = code-ISTORE;
            return arg+(x&1);
        }
        if (code == RET)
            return arg;
        return -1;
    }
    public void emitTo(DataOutputStream o) throws IOException {
        super.emitTo(o);
        o.writeByte(arg);
    }
    public int size() { return 2; }
    public String toStringThis() {
        return codenames[code]+" "+arg;
    }
    public boolean equals(Object other) {
        if (!(other instanceof ByteArg)) return false;
        ByteArg b = (ByteArg)other;
        if (arg != b.arg) return false;
        return super.equals(other);
    }
}

/** Concrete class for instructions that take a short argument.
 **/
class ShortArg extends Instr {
    int arg;
    ShortArg(int code, int arg) {
        super(code);
        this.arg = arg;
    }
    protected int getLocalIndex() {
        if (code == IINC)
            return arg>>>8;
        return -1;
    }
    public void emitTo(DataOutputStream o) throws IOException {
        super.emitTo(o);
        o.writeShort(arg);
    }
    public int size() { return 3; }
    public boolean equals(Object other) {
        if (!(other instanceof ShortArg)) return false;
        ShortArg b = (ShortArg)other;
        if (arg != b.arg) return false;
        return super.equals(other);
    }
}

/** Concrete class for instructions that use the constant pool.
 **/
class ConstantArg extends Instr {
    Constant arg;
    ConstantArg(int code, Constant arg) {
        super(code);
        this.arg = arg;
    }

    public void prepareForEmit(ConstantPool c) {
        arg = c.addConstant(arg);
    }

    public void emitTo(DataOutputStream o) throws IOException {
        /* LDC is special, it changes if the constant index is too large */
        if (code == LDC1 ||
            (code == LDC1_W && arg.index < 256)) {
            code = LDC1;
            super.emitTo(o);
            arg.emitByteIndex(o);
        } else {
            super.emitTo(o);
            arg.emitIndex(o);           
        }
    }
    public int size() { 
        if (arg == null)
            return 3;
        if (code == LDC1 ||
            (code == LDC1_W && arg.index < 256)) {
            return 2; 
        } else {
            return 3;
        }
    }
    public boolean equals(Object other) {
        if (!(other instanceof ConstantArg)) return false;
        ConstantArg b = (ConstantArg)other;
        if (!arg.equals(b.arg)) return false;
        return super.equals(other);
    }
    public String toStringThis() {
        return super.toStringThis()+": "+arg;
    }
}

/** Concrete class for instructions that has a simple label (jsr, if, goto)
 **/
class LabelArg extends ShortArg {
    Instr target;
    LabelArg(int code, Instr target) {
        super(code, -1);
        this.target = target;
    }
    public void emitTo(DataOutputStream o) throws IOException {
        arg = target.offset - offset;
        super.emitTo(o);
        if (target.offset == -1) 
            throw new Error("Unfixed target "+target+" for "+this);
    }
    public int size() { return 3; }
    public String toStringThis() {
        return codenames[code]+" "+target.offset;
    }
    /* This might not really work */
    public boolean equals(Object other) {
        if (!(other instanceof LabelArg)) return false;
        LabelArg b = (LabelArg)other;
        if (!target.equals(b.target)) return false;
        return super.equals(other);
    }
}

/** Concrete class for instructions that take a constant + a byte argument.
 **/
class ConstantByteArg extends Instr {
    Constant constarg;
    int bytearg;
    ConstantByteArg(int code, Constant constarg, int bytearg) {
        super(code);
        this.constarg = constarg;
        this.bytearg = bytearg;
    }
    public void prepareForEmit(ConstantPool c) {
        constarg = c.addConstant(constarg);
    }
    public void emitTo(DataOutputStream o) throws IOException {
        super.emitTo(o);
        constarg.emitIndex(o);
        o.writeByte(bytearg);
    }
    public int size() { return 4; }
    public boolean equals(Object other) {
        if (!(other instanceof ConstantByteArg)) return false;
        ConstantByteArg b = (ConstantByteArg)other;
        if (bytearg != b.bytearg) return false;
        if (!constarg.equals(b.constarg)) return false;
        return super.equals(other);
    }
}

/** Concrete class for the tableswitch instruction 
 **/
class TableSwitch extends Instr {
    Instr defaultcase;
    int min;
    int max;
    MonoList targets;

    TableSwitch(Instr defaultcase, int min, int max, MonoList targets) {
        super(TABLESWITCH);
        this.defaultcase = defaultcase;
        this.min = min;
        this.max = max;
        this.targets = targets;
    }

    public void emitTo(DataOutputStream o) throws IOException {
        super.emitTo(o);
        for (int i = 0; i < 3-(offset%4); i++) {
            o.writeByte(0);
        }

        o.writeInt(defaultcase.offset-this.offset);
        o.writeInt(min);
        o.writeInt(max);
        Iterator iter = targets.iterator();
        for (int i = min; i <= max; i++) {
            o.writeInt(((Instr)iter.next()).offset-this.offset);
        }
        for (int i = 0; i < offset%4; i++) {
            o.writeByte(0);
        }
    }
    public int size() {
        if (targets == null) return 4+12;
        return 4+12+4*targets.size();
    }
}

/** Concrete class for the lookupswitch instruction
 **/
class LookupSwitch extends Instr {
    Instr defaultcase;
    SortedMap cases;

    LookupSwitch(Instr defaultcase, SortedMap cases) {
        super(LOOKUPSWITCH);
        this.defaultcase = defaultcase;
        this.cases = cases;
    }

    public void emitTo(DataOutputStream o) throws IOException {
        super.emitTo(o);
        for (int i = 0; i < 3-(offset%4); i++) {
            o.writeByte(0);
        }
        o.writeInt(defaultcase.offset-this.offset);
        o.writeInt(cases.size());
        for (Iterator i = cases.entrySet().iterator(); i.hasNext(); ) {
            Map.Entry entry = (Map.Entry)i.next();
            o.writeInt(((Integer)entry.getKey()).intValue());
            Instr target = (Instr)entry.getValue();
            o.writeInt(target.offset-this.offset);
        }
        for (int i = 0; i < offset%4; i++) {
            o.writeByte(0);
        }
    }
    public int size() {
        if (cases == null) return 8;
        return 4+4+4+8*cases.size();
    }
}

class InterfaceInstr extends Instr {
    Constant methodconstant;
    InterfaceInstr(Constant methodconstant) {
        super(INVOKEINTERFACE);
        this.methodconstant = methodconstant;
    }

    public void prepareForEmit(ConstantPool c) {
        methodconstant = c.addConstant(methodconstant);
    }

    public void emitTo(DataOutputStream o) throws IOException {
        super.emitTo(o);
        methodconstant.emitIndex(o);
        o.writeByte(((ConstantInterfaceMethodRef)methodconstant).numArgs());
        o.writeByte(0);
    }
    public int size() { return 5; }
}

/* End of class Instr*/
