/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * Test.java
 *
 *
 */

package uiuc.Jumbo.Jaemus;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
/**
 * A class for performing unit tests.
 * Each class should have a public void unitTest() method.
 * Inside this method, Test.assertEquals(o, o) will check if two objects
 * are the same, and output appropriate error messages.
 * The class must be listed in the test_suite array.
 * @version      1.0,  2 Oct 2001
 * @author       Lars Clausen
 *
*/

public class Test {
    static int test_num;
    static String class_name;
    static int failures;
    static Class[] void_args = new Class[0];
    static int total_tests;

    static String[] test_suite = 
    { "uiuc.Jumbo.Jaemus.Type",
      "uiuc.Jumbo.Jaemus.MethodType"
    };

    public static void assertEquals(Object o1, Object o2) {
        if (!(o1.equals(o2))) {
            System.out.println(class_name+":Test "+test_num+":"+o1+" != "+o2);
            failures++;
        }
        test_num++;
    }

    public static void testClass(String classname) {
        test_num = 0;
        class_name = classname;
        try {
            Class cl = Class.forName(classname);
            Method testmethod = cl.getDeclaredMethod("unitTest", void_args);
            try {
                testmethod.invoke(null, void_args);
            } catch (Throwable e) {
                System.out.println("Exception outside unitTest(): "+e);
                failures++;
            }
        } catch (Exception e) {
            System.out.println("Error in test suite: "+e);
            failures++;
        }
        total_tests += test_num;
    }

    public static void runFunctionalTests() {
        int i = 0;
        try {
            Class c = Class.forName("uiuc.Jumbo.Jaemus.Test");
            Class[] args = new Class[0];
            while (true) {
                Method m = c.getDeclaredMethod("functionalTest"+i, args);
                total_tests++;
                try {
                    m.invoke(null, args);
                } catch (InvocationTargetException e) {
                    System.out.println("Exception while executing "+m+": "+e);
                    e.printStackTrace();
                    failures++;
                }               
                i++;
            }
        } catch (NoSuchMethodException e) {
        } catch (ClassNotFoundException e) {
            System.out.println("Failed to find self class: "+e);
        } catch (IllegalAccessException e) {
            System.out.println("Can't get to method `functionalTest"+i+"': "+e);
        } 
    }

    public static void main(String argv[]) {
      /* Unit Tests */
        failures = 0;
        total_tests = 0;
        for (int i = 0; i < test_suite.length; i++) {
            testClass(test_suite[i]);
        }
        if (failures > 0) {
            System.out.println(failures+" failures in "+total_tests+" tests.");
            System.exit(1);
        } else {
            System.out.println(total_tests+" tests passed for "+test_suite.length+" classes");
        }

        failures = 0;
        total_tests = 0;
        /* Functional tests */
        
        runFunctionalTests();

        if (failures > 0) {
          System.out.println(failures+" failures in "+total_tests+" tests.");
          System.exit(1);
        } else {
          System.out.println("All "+total_tests+" functional tests ok.");
        }
    }

    private static MethodInfo makeTestMethod(String classname, String methodname) {
        ClassInfo cl = new ClassInfo(classname, "");
        cl.flags = Flags.PUBLIC;
        MethodInfo m = new MethodInfo(cl, Flags.PUBLIC|Flags.STATIC, methodname, 
                                      new MethodType(Type.int_type));
        return m;
    }

    private static void testMethod(MethodInfo mi, int result) {
        ClassInfo cl = mi.inClass;
        cl.addMethod(mi);
        try {
            cl.writeClassFile();
            Class c = Class.forName(cl.getJavaName());
            if (c == null) {
                System.out.println("Failed to load "+cl.getJavaName());
                failures++;
                return;
            }
            Method m = c.getDeclaredMethod(mi.getName(), new Class[0]);
            if (m == null) {
                System.out.println("Method "+mi.getName()+" not in class "+cl.getJavaName());
                failures++;
                return;
            }
            int i = ((Integer)m.invoke(null, new Object[0])).intValue();
            if (i != result) {
                System.out.println("Method "+cl.getJavaName()+"."+mi.getName()+"() didn't return "+result+": "+i);
                failures++;
                return;
            }
            new File(cl.name+".class").delete();
        } catch (Exception e) {
            System.out.println("Exception creating "+cl.getJavaName()+": "+e);
            e.printStackTrace();
            failures++;
        }

    }

    public static void functionalTest0() {
        ClassInfo cl = new ClassInfo("Tests/Test0", "");
        try {
            cl.writeClassFile();
            Class c = Class.forName("Tests.Test0");
            if (c == null) {
                System.out.println("Failed to load Tests.Test0");
                failures++;
            }
            new File("Tests/Test0.class").delete();
        } catch (Exception e) {
            System.out.println("Exception creating Tests.Test0: "+e);
            e.printStackTrace();
            failures++;
        }
    }
    public static void functionalTest1() {
        ClassInfo cl = new ClassInfo("Tests/Test1", "");
        cl.addMethod(new MethodInfo(cl, Flags.ABSTRACT, "testMethod", 
                                    new MethodType(Type.void_type)));
        try {
            cl.writeClassFile();
            Class c = Class.forName("Tests.Test1");
            if (c == null) {
                System.out.println("Failed to load Tests.Test1");
                failures++;
                return;
            }
            if (c.getDeclaredMethod("testMethod", new Class[0]) == null) {
                System.out.println("Method testMethod() not in class Tests.Test1\n");
                failures++;
                return;
            }
            new File("Tests/Test1.class").delete();
        } catch (Exception e) {
            System.out.println("Exception creating Tests.Test1: "+e);
            e.printStackTrace();
            failures++;
        }
    }
    public static void functionalTest2() {
        MethodInfo mi = makeTestMethod("Tests/Test2", "testMethod");
        Instr i = Instr.genIconst(23).addReturn(Type.int_type);
        mi.addCode(i);
        testMethod(mi, 23);
    }
    public static void functionalTest3() {
        MethodInfo mi = makeTestMethod("Tests/Test3", "testMethod");
        Instr instr = Instr.genIconst(23).
            addIconst(19).
            addAdd(Type.int_type).
            addReturn(Type.int_type);
        mi.addCode(instr);
        testMethod(mi, 42);
    }
    public static void functionalTest4() {
        MethodInfo mi = makeTestMethod("Tests/Test4", "testMethod");

        Instr ifend = Instr.genNil("End of if");
        Instr iffalse = Instr.genIconst(69).addGoto(ifend);
        Instr iftrue = Instr.genIconst(87);
        Instr ifhead = Instr.genIconst(23).
            addIf(Instr.CMP_EQ, iftrue).
            append(iffalse).
            append(iftrue).
            append(ifend).
            addReturn(Type.int_type);
        mi.addCode(ifhead);

        testMethod(mi, 69);
    }
    public static void functionalTest5() {
        MethodInfo mi = makeTestMethod("Tests/Test5", "testMethod");
        Instr i = Instr.genGetstatic("java/lang/System", "out", new Type("java/io/PrintStream"));
        i.addIconst(10);
        i.addInvokevirtual("java/io/PrintStream", "println", new MethodType(Type.void_type, Type.int_type));
        i.addIconst(22);
        i.addReturn(Type.int_type);
        mi.addCode(i);
    
        testMethod(mi, 22);
    }
    public static void functionalTest6() {
        MethodInfo mi = makeTestMethod("Tests/Test6", "testMethod");
        Instr i = Instr.genConst(50).
            addConst(1).
            addBitwise(Instr.USHR, Type.int_type).
            addReturn(Type.int_type);
        mi.addCode(i);
        testMethod(mi, 25);
    }
    public static void functionalTest7() {
        MethodInfo mi = makeTestMethod("Tests/Test7", "testMethod");
        Instr i = Instr.genConst(40).addReturn(Type.int_type);
        Instr h = Instr.genConst(30).addReturn(Type.int_type);
        i = Instr.addHandler(i, h, new Type("java/lang/Throwable")).append(h);
        mi.addCode(i);
        testMethod(mi, 40);
    }
    public static void functionalTest8() {
        MethodInfo mi = makeTestMethod("Tests/Test8", "testMethod");
        Instr i = 
            Instr.genNew("java/lang/Error").
            addDup(1, 0).
            addInvokenonvirtual("java/lang/Error", "<init>", new MethodType(Type.void_type)).
            addThrow().
            addConst(40).
            addReturn(Type.int_type);
        Instr h = Instr.genConst(30).addReturn(Type.int_type);
        i = Instr.addHandler(i, h, new Type("java/lang/Throwable")).append(h);
        mi.addCode(i);
        testMethod(mi, 30);
    }
    /* Test the switch statement */
    public static void functionalTest9() {
        MethodInfo mi = makeTestMethod("Tests/Test9", "testMethod");
        Instr case1 = Instr.genConst(10).addReturn(Type.int_type);
        Instr case2 = Instr.genConst(20).addReturn(Type.int_type);
        Instr case3 = Instr.genConst(30).addReturn(Type.int_type);
        Instr def = Instr.genConst(40).addReturn(Type.int_type);
        SortedMap cases = new TreeMap();
        cases.put(new Integer(1), case1);
        cases.put(new Integer(2), case2);
        cases.put(new Integer(3), case3);
        Instr i = Instr.genConst(1).
            addSwitch(cases, def).
            append(case1).
            append(case2).
            append(case3).
            append(def);
        mi.addCode(i);
        testMethod(mi, 10);

    }
    public static void functionalTest10() {
        MethodInfo mi = makeTestMethod("Tests/Test10", "testMethod");
        Instr case1 = Instr.genConst(10).addReturn(Type.int_type);
        Instr case2 = Instr.genConst(20).addReturn(Type.int_type);
        Instr case3 = Instr.genConst(30).addReturn(Type.int_type);
        Instr def = Instr.genConst(40).addReturn(Type.int_type);
        SortedMap cases = new TreeMap();
        cases.put(new Integer(1), case1);
        cases.put(new Integer(10), case2);
        cases.put(new Integer(100), case3);
        Instr i = Instr.genConst(1).
            addSwitch(cases, def).
            append(case1).
            append(case2).
            append(case3).
            append(def);
        mi.addCode(i);
        testMethod(mi, 10);

    }
    public static void functionalTest11() {
        ClassInfo ci = ClassTable.get("java/lang/String");
        if (ci == null) {
            failures++;
            System.out.println("Failed to load class java/lang/String ");
        }
    }

}
