/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * TestMethodType.java
 *
 *
 */

package uiuc.Jumbo.Jaemus;

import junit.framework.TestCase;

import uiuc.Jumbo.Jaemus.*;
import java.util.*;

public class TestMethodType extends TestCase {

    public TestMethodType(String name) {
        super(name);
    }

    public void testVoidReturnType() {
        checkNoArgMethod( Type.void_type );
        checkOneArgMethod( Type.void_type, Type.int_type );
        checkTwoArgMethod( Type.void_type, Type.int_type, Type.int_type );
        checkThreeArgMethod( Type.void_type, Type.int_type, Type.int_type, 
                             Type.int_type );
        checkManyArgMethod( Type.void_type, 
                            new Type[] { Type.int_type, Type.int_type } );
    }

    public void testNoArgs() {
        checkNoArgMethod( Type.float_type );
        checkNoArgMethod( Type.object_type );
        checkNoArgMethod( Type.string_type );
    }

    public void testOneArg() {
        checkOneArgMethod( Type.int_type, Type.boolean_type );
        checkOneArgMethod( Type.double_type, Type.float_type );
        checkOneArgMethod( Type.string_type, Type.string_type );
    }

    public void testTwoArg() {
        checkTwoArgMethod( Type.int_type, Type.boolean_type, Type.string_type );
        checkTwoArgMethod( Type.double_type, Type.float_type, Type.short_type );
        checkTwoArgMethod( Type.long_type, Type.string_type, Type.object_type );
        checkTwoArgMethod( Type.string_type, Type.string_type, 
                           Type.string_type );
    }

    public void testThreeArg() {
        checkThreeArgMethod( Type.object_type, Type.string_type, 
                             Type.long_type, Type.object_type );
        checkThreeArgMethod( Type.float_type, Type.double_type, 
                             Type.double_type, Type.object_type );
        checkThreeArgMethod( Type.int_type, Type.short_type, 
                             Type.byte_type, Type.boolean_type );
    }

    public void testManyArgs() {
        checkManyArgMethod( Type.int_type, 
                            new Type[] { Type.object_type, Type.object_type, 
                                         Type.object_type, Type.string_type, 
                                         Type.string_type, Type.long_type, 
                                         Type.boolean_type, Type.float_type, 
                                         Type.double_type, Type.char_type } );
    }

    /********************* Convenience methods. ******************/

    private void checkNoArgMethod( Type return_type ) {
        checkMethod( new MethodType(return_type), 
                     "()"+return_type.bytecodeString() );
    }

    private void checkOneArgMethod( Type return_type, Type arg1 ) {
        checkMethod( new MethodType(return_type, arg1), 
                     "(" + arg1.bytecodeString() + ")" + 
                        return_type.bytecodeString() );
    }

    private void checkTwoArgMethod( Type return_type, Type arg1, Type arg2 ) {
        checkMethod( new MethodType(return_type, arg1, arg2), 
                     "(" + arg1.bytecodeString() + 
                         arg2.bytecodeString() + ")" + 
                         return_type.bytecodeString() );
    }

    private void checkThreeArgMethod( Type return_type, 
                                      Type arg1, Type arg2, Type arg3 ) {
        checkMethod( new MethodType(return_type, arg1, arg2, arg3), 
                     "(" + arg1.bytecodeString() + 
                         arg2.bytecodeString() + 
                         arg3.bytecodeString() + ")" + 
                         return_type.bytecodeString() );
    }

    private void checkManyArgMethod( Type return_type, Type[] args ) {
        String code = "(";

        for(int i = 0; i < args.length; i++) {
            code += args[i].bytecodeString();
        }
        code += ")" + return_type.bytecodeString();

        checkMethod( new MethodType(return_type, args), code );
    }

    private void checkMethod( MethodType mt, String code ) {
        checkMethod( mt, code, "" );
    }

    private void checkMethod( MethodType mt, String code, String message ) {
        assertEquals( ""+mt+"; "+message+"\n", code, mt.bytecodeString() );
    }
}
