/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents an array initializer (e.g., <code>{ {1, 2}, {3, 4} }</code>.
 *
 * Not really an expression, but best to think of it that way.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;

import uiuc.Jumbo.Util.*;

public class ArrayInitializer extends Expression  {
    /**
     * @param i  list of expressions with which to initialize an array
     */
    public ArrayInitializer( List i ) {
        initializers = i;
        setParents(initializers);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        return replaceInList(initializers, old_child, new_child);
    }

    public void setType(Type t) {
        type = t;
    }

    public void toSource() {
        if (type != null) {
            print("new ");
            type.toSource();
        }
        ocurly();
        printSourceList(initializers, ", ");
        ccurly();
    }

    public IRList toIRList(List initializers, Type t) {
        MonoList l = new ArrayMonoList();
        if (initializers != null)
            for (Iterator i = initializers.iterator(); i.hasNext(); ) {
                l.add(((Expression)i.next()).toIR(t));
            }
        return IR.make(l);
    }

    public IR toIR() {
        if (type == null) throw new Error("toIR() called on untyped initializeArray.");
        return toIR(type);
    }

    public IR toIR(Type t) {
        return IR.make("initializeArray", new ArrayMonoList()
                       .add(t.toIR())
                       .add(toIRList(initializers, t.getElementType())));
    }

    public Type getType(ClassLoader loader) {
        if (type == null) throw new Error("Don't know type of array initializer yet");
        return type;
    }

    /**
     * List of expressions to initialize an array with.
     */
    protected List initializers;

    /**
     * The type of the array created.
     */
    protected Type type;
}
