/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents a type cast.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.ArrayMonoList;


public class CastExpression extends Expression {
   /**
    * @param t    type to cast e to
    * @param e    given expression to typecast
    */
   public CastExpression( Type t, Expression e ) { 
       setType(t);
       setExpression(e);
   }

   protected boolean replaceUC(Node old_child, Node new_child) {
       if (old_child == type) {
           setType((Type)new_child);
           return true;
       }
       else if (old_child == expression) {
           setExpression((Expression)new_child);
           return true;
       }
       else {
           return false;
       }
   }

   private void setType(Type t) {
       type = t;
       setParent(type);
   }

   private void setExpression(Expression e) {
       expression = e;
       setParent(expression);
   }

   public IR toIR() {
       return IR.make("cast", new ArrayMonoList()
                        .add(expression.toIR())
                        .add(type.toIR()));
   }

    public void toSource() {
        print("((");
        type.toSource();
        print(")");
        expression.toSource();
        print(")");
    }

    public Type getType(ClassLoader loader) {
        return type;
    }

   /**
    * Type to cast expression to.
    */
   protected Type type;
   /**
    * Expression to typecast.
    */
   protected Expression expression;
}
