/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*; 
import uiuc.Jumbo.Jaemus.*; 

/**
 * Represents a catch statement.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class CatchStatement extends Statement {
   /**
    * @param p    parameter to the catch block
    * @param b    body of the catch statement
    */
   public CatchStatement( Parameter p, StatementList b ) { 
       setParameter(p);
       setBody(b);
   }

   protected boolean replaceUC(Node old_child, Node new_child) {
       if (parameter == old_child) {
           setParameter((Parameter)new_child);
           return true;
       }
       else if (body == old_child) {
           setBody((StatementList)new_child);
           return true;
       }
       else {
           return false;
       }
   }

   private void setParameter(Parameter p) {
       parameter = p;
       setParent(parameter);
   }

   private void setBody(StatementList b) {
       body = b;
       setParent(b);
   }

    public void toSource() {
        print(" catch (");
        parameter.toSource();
        print(") ");
        body.toSource();
    }

    public IR toIR() {
        return IR.make(parameter.is_final?Flags.FINAL:0,
                       parameter.type.toType(),
                       parameter.variable.name.toIRString(),
                       (IRCode)body.toIR());
    }

   /**
    * Parameter to the catch block.
    */
   protected Parameter parameter;
   /**
    * Body of the catch statement.
    */
   protected StatementList body;
}
