/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * Represents an character literal. 
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class CharacterLiteral extends Literal {
    /**
     * @param v   value of the literal
     */
    public CharacterLiteral( Character v ) { 
        super( "characterConstant", v );
    }

    public IR toIR() {
        return IR.make("characterConstant", new ArrayMonoList()
                        .add(IR.make(((Character)value).charValue())));
    }

    public void printChar() {
        char c = ((Character)value).charValue();
        switch (c) {
            case '\\': case '\'': case '\"':
                                  print( "\'\\"+c+"\'" ); break;
            case '\b': print("\'\\b\'"); break;
            case '\t': print("\'\\t\'"); break;
            case '\n': print("\'\\n\'"); break;
            case '\f': print("\'\\f\'"); break;
            case '\r': print("\'\\r\'"); break;
            default:
                       print( "'" + c + "'" );
        }
    }

    public void toSource() {
        printChar();
    }
    public int compare(Literal l) {
        return ((Character)value).compareTo((Character)l.value);
    }
    public boolean equals(Object o) {
        if (o instanceof CharacterLiteral) {
            return value.equals(((CharacterLiteral)o).value);
        }
        return false;
    }
    public Type getType(ClassLoader loader) {
        return new Type("char");
    }
}
