/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.ArrayMonoList;

/**
 * Represents a class instance creation.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ClassInstanceCreation extends Expression {
   /**
    * @param c  name of the class being instanced
    * @param a  optional argument list
    */
   public ClassInstanceCreation( Identifier c, List a ) { 
       setClassName(c);
       arguments = a;
       setParents(arguments);
   }

   protected boolean replaceUC(Node old_child, Node new_child) {
       if (class_name == old_child) {
           setClassName((Identifier)old_child);
           return true;
       }
       else {
           return replaceInList(arguments, old_child, new_child);
       }
   }

   private void setClassName(Identifier c) {
       class_name = c;
       setParent(class_name);
   }

   public IR toIR() {
       return IR.make("newObject", new ArrayMonoList()
                      .add(IR.make(class_name.toString()))
                      .add(toIRList(arguments)));
   }

    public void toSource() {
        print("new ");
        class_name.toSource();
        print("(");
        printSourceList(arguments, ", ");
        print(")");
    }

    public String toString() {
        String r = "new "+class_name+"(";
        if (arguments != null) 
            for (Iterator i = arguments.iterator(); i.hasNext(); ) {
                r += i.next().toString();
                if (i.hasNext()) r += ", ";
            }
        return r+")";
    }

    public Type getType(ClassLoader loader) {
        return new Type(class_name);
    }

   /**
    * Name of the class of the instance being created
    */
   protected Identifier class_name;

   /**
    * Arguments.
    */
   protected List arguments;
}
