/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * Represents a double literal. 
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class DoubleLiteral extends Literal {
    /**
     * @param v   value of the literal
     */
    public DoubleLiteral( Number v ) { 
        super( "doubleConstant", v );
        double f = v.doubleValue();
    }

    public IR toIR() {
        return IR.make("doubleConstant", new ArrayMonoList()
                        .add(IR.make(((Double)value).doubleValue())));
    }

    public String typeMarker() {
        return (value instanceof Double?"d":"");
    }

    public void toSource() {
        print(value+"d");
    }

    public int compare(Literal l) {
        double n = ((Double)l.value).doubleValue();
        double thisn = ((Double)value).doubleValue();
        return (thisn<n?-1:thisn>n?1:0);
    }
    public boolean equals(Object o) {
        if (o instanceof DoubleLiteral) {
            return value.equals(((DoubleLiteral)o).value);
        }
        return false;
    }
    public Type getType(ClassLoader loader) {
        return new Type("double");
    }
}
