/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Generic base class for any kind of simple expression.
 *
 * Specifically, expressions whose constructors are string 
 * constants.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

interface ClassLoader extends TypeEnv {
    ClassDefiner getClass(String name);
    Type getVarType(Identifier i);
}

public abstract class Expression extends Node  {
    /**
     * Creates an empty expression.
     */
    public Expression() {}

    /**
     * Creates an expression with the constructor string given.
     *
     * @param c  string of constructors representing this expression
     */
    public Expression( String c ) { 
        super( c );
    }

    /** Creates an IR for an expression for initializing a variable of the
     * given type.
     */
    public IR toIR(Type t) {
        return toIR();
    }

    /** Returns the type of this expression.
     * @param loader A object that allows us to load classes as needed.
     */
    public abstract Type getType(ClassLoader loader);
}
