/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a field access.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class FieldAccess extends Expression  {
    /**
     * @param t   target of the field access
     * @param f   field being accessed
     */
    public FieldAccess( Expression t, Identifier f ) { 
        constructor = "Field";
        setTarget(t);
        setField(f);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == target) {
            setTarget((Expression)new_child);
            return true;
        }
        else if (old_child == field) {
            setField((Identifier)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setTarget(Expression t) {
        target = t;
        setParent(target);
    }

    private void setField(Identifier f) {
        field = f;
        setParent(field);
    }

    public void toSource() {
        target.toSource();
        print(".");
        field.toSource();
    }

    public IR toIR() {
        return IR.make("get"+constructor, new ArrayMonoList()
                       .add(target.toIR())
                       .add(field.toIR()));
    }

    public Type getType(ClassLoader loader) {
        if (field.isSimple() && field.getField(0).toString().equals("length")) {
            return new Type("int");
        }
        ClassDefiner cd = loader.getClass(target.getType(loader).getName());
        if (cd == null) {
            System.out.println("Found no class for "+target.getType(loader));
            return null;
        }
        Variable v = cd.findField(field.toString());
        return v.type;
    }

    public String toString() {
        return target+"."+field;
    }

    /**
     * Target of the field access.
     */
    protected Expression target;

    /**
     * Field being accessed.
     */
    protected Identifier field;
}
