/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a for statement.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ForStatement extends WhileStatement {
    /**
     * @param init optional initialization statement(s)
     * @param te   optional test expression
     * @param ud   optional update statement(s)
     * @param b    body statement of the for loop
     */
    public ForStatement( Statement init, Expression te, 
            Statement ud, Statement b ) { 
        super( te, b );

        setInitialize(init);
        setUpdate(ud);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (super.replaceUC(old_child, new_child)) {
            return true;
        }
        else if (old_child == initialize) {
            setInitialize((Statement)new_child);
            return true;
        }
        else if (old_child == update) {
            setUpdate((Statement)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setInitialize(Statement init) {
        if ( init instanceof LocalVariableDeclaration ) {
            initialize = new StatementList();
            ((StatementList)initialize).appendStatement( init );
            setParent(initialize);
        }
        else {
            initialize = init;
            setParent(initialize);
        }
    }

    private void setUpdate(Statement ud) {
        update = ud;
        setParent(update);
    }

    public IR toIR() {
        return IR.make("forLoop", new ArrayMonoList()
                       .add(initialize!=null?initialize.toIR():IR.makeNoop())
                       .add(test!=null?test.toIR():IR.makeNoop())
                       .add(update!=null?update.toIR():IR.makeNoop())
                       .add(body!=null?body.toIR():IR.makeNoop()));
    }

    public IR toIR(String label) {
        return IR.make("forLabel", new ArrayMonoList()
                       .add(IR.make(label))
                       .add(initialize!=null?initialize.toIR():IR.makeNoop())
                       .add(test!=null?test.toIR():IR.makeNoop())
                       .add(update!=null?update.toIR():IR.makeNoop())
                       .add(body!=null?body.toIR():IR.makeNoop()));
    }

    public void toSource() {
        print("for (");
        if ( initialize != null ) {
            if (initialize instanceof StatementList)
                ((StatementList)initialize).toCommaSource();
            else
                initialize.toSource();
        }
        print("; ");
        if ( test != null ) {
            test.toSource();
        }
        print("; ");
        if (update != null) {
            if (update instanceof StatementList)
                ((StatementList)update).toCommaSource();
            else
                update.toSource();
        }
        print(") ");
        if (body instanceof EmptyStatement)
            print(";");
        else {
            body.toSource();
        }
    }

    public boolean needsSemicolon() { return false; }

    /**
     * Initialization statement(s).
     */
    protected Statement initialize;
    /**
     * Update statement(s).
     */
    protected Statement update;
}
