/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * Represents an if-then statement with an else clause
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class IfThenElseStatement extends Statement {
    /**
     * @param tst   test expression
     * @param thn   'then' branch
     * @param els   optional 'else' branch
     */
    public IfThenElseStatement( Expression tst, 
            Statement thn, Statement els ) { 
        setTest(tst);
        setThenBranch(thn);
        setElseBranch(els);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == test) {
            setTest((Expression)new_child);
            return true;
        }
        else if (old_child == then_branch) {
            setThenBranch((Statement)new_child);
            return true;
        }
        else if (old_child == else_branch) {
            setElseBranch((Statement)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setTest(Expression tst) {
        test = tst;
        setParent(test);
    }

    private void setThenBranch(Statement thn) {
        then_branch = thn;
        setParent(then_branch);
    }

    private void setElseBranch(Statement els) {
        else_branch = els;
        setParent(else_branch);
        constructor = (els == null?"ifThen":"ifThenElse");
    }

    public IR toIR() {
        MonoList args = new ArrayMonoList()
                        .add(test.toIR())
                        .add(then_branch.toIR());
        if (else_branch != null) {
            args.add(else_branch.toIR());
        }
        return IR.make(constructor, args);
    }

    public void toSource() {
        print("if (");
        test.toSource();
        print(") ");
        then_branch.toSource();
        if (then_branch.needsSemicolon()) print(";");
        if (else_branch != null) {
            print(" else ");
            else_branch.toSource();
            if (else_branch.needsSemicolon()) print(";");
        }
    }
    public boolean needsSemicolon() { return false; }

    /**
     * Test expression.
     */
    protected Expression test;
    /**
     * "Then" branch.
     */
    protected Statement then_branch;
    /**
     * "Else" branch.
     */
    protected Statement else_branch;
}
