/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a incrementary expression (<code>i++</code> or 
 * <code>--i</code>, for instance).
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class IncrementExpression extends UnaryExpression {
    /**
     * @param constr     constructor to use
     * @param i          Integer amount to postinc/predec by
     * @param e          Expression being operated upon
     */
    public IncrementExpression( String constr, int i, Expression e ) { 
        super(constr, e);
        amount = i;

        if ( expression instanceof ArrayAccess ) {
            constructor = "arrayP" + constructor.substring(1);
        }
    }

    public void toSource() {
        if (constructor.startsWith("pre") ||
                constructor.startsWith("arrayPre")) {
            if (amount == -1) print("--");
            else print("++");
            expression.toSource();
        } else {
            expression.toSource();
            if (amount == -1) print("--");
            else print("++");
        }
    }

    public IR toIR() {
        IRCode ir = IR.make(constructor, new ArrayMonoList());
        if (expression instanceof QualifiedNameExpression) {
            QualifiedNameExpression qne = (QualifiedNameExpression)expression;
            if (qne.name.isSimple()) {
                ir.addArg(IR.makeNull("uiuc.Jumbo.Compiler.Code"));
                ir.addArg(qne.name.toIRString());
            } else {
                throw new Error("Can't increment non-simple QNE "+qne+" yet");
            }
        } else if (expression instanceof FieldAccess) {
            FieldAccess fa = (FieldAccess)expression;
            ir.addArg(fa.target.toIR());
            ir.addArg(fa.field.toIR());
        } else if (expression instanceof ArrayAccess) {
            ArrayAccess aa = (ArrayAccess)expression;
            ir.addArg(aa.target.toIR());
            ir.addArg(aa.index.toIR());
        } else {
            ir.addArg(IR.makeNull("uiuc.Jumbo.Compiler.Code"));
            ir.addArg(expression.toIR());
        }

        ir.addArg(IR.make(amount));
        return ir;
    }

    /**
     * Amount to increment/decrement by.
     */
    protected int amount;
}
