/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * Represents an instanceof expression.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class InstanceOfExpression extends Expression  {
    /**
     * @param e          Expression
     * @param t          Type to test e against
     */
    public InstanceOfExpression( Expression e, Type t ) { 
        super("instanceOf");
        setExpression(e);
        setType(t);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == expression) {
            setExpression((Expression)new_child);
            return true;
        }
        else if (old_child == type) {
            setType((Type)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setExpression(Expression e) {
        expression = e;
        setParent(expression);
    }

    private void setType(Type t) {
        type = t;
        setParent(type);
    }

    public IR toIR() {
        return IR.make(constructor, new ArrayMonoList()
                        .add(expression.toIR())
                        .add(type.toIR()));
    }

    public void toSource() {
        print("(");
        expression.toSource();
        print(" instanceof ");
        type.toSource();
        print(")");
    }

    public Type getType(ClassLoader loader) {
        return new Type("boolean");
    }

    /**
     * Expression to be tested.
     */
    protected Expression expression;
    /**
     * Type to test expression against.
     */
    protected Type type;
}
