/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * Represents an integer literal. 
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class IntegerLiteral extends Literal {
    /**
     * @param v   value of the literal
     */
    public IntegerLiteral( int v ) {
        super( "integerConstant", new Integer( v ) );
    }

    /**
     * @param v   value of the literal
     */
    public IntegerLiteral( Number v ) { 
        super( "integerConstant", v );
    }

    public IR toIR() {
        return IR.make("integerConstant", new ArrayMonoList()
                        .add(IR.make(((Integer)value).intValue())));
    }

    public int compare(Literal l) {
        int n = ((Integer)l.value).intValue();
        int thisn = ((Integer)value).intValue();
        return (thisn<n?-1:thisn>n?1:0);
    }
    public boolean equals(Object o) {
        if (o instanceof IntegerLiteral) {
            return value.equals(((IntegerLiteral)o).value);
        }
        return false;
    }

    public Type getType(ClassLoader loader) {
        return new Type("int");
    }

    public void toSource() {
        print(value);
    }
    public String toString() {
        return value+"@"+Integer.toHexString(System.identityHashCode(value));
    }
}
