/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a labeled statement.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class LabeledStatement extends Statement implements Labelable {
    /**
     * @param l  label to be attached to s
     * @param s  statement to be labeled
     */
    public LabeledStatement( String l, Statement s ) { 
        label = l;
        setStatement(s);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == statement) {
            setStatement((Statement)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setStatement(Statement s) {
        statement = s;
        setParent(statement);
    }

    public IR toIR() {
        if ( statement instanceof Labelable ) {
            return ((Labelable)statement).toIR(label);
        } else {
            return IR.make("blockLabel", new ArrayMonoList()
                           .add(IR.make(label))
                           .add(statement.toIR()));
        }
    }

    // This should really allow for multiple labels on a single loop,
    // but we don't support that.  Yet.
    public IR toIR(String label) {
        if ( statement instanceof Labelable ) {
            return ((Labelable)statement).toIR(label);
        } else {
            return IR.make("blockLabel", new ArrayMonoList()
                           .add(IR.make(label))
                           .add(statement.toIR()));
        }
    }

    public void toSource() {
        print(label+": ");
        statement.toSource();
        if (statement.needsSemicolon()) print("; ");
    }

    public boolean needsSemicolon() { return false; }

    public String getLabel() {
        return label;
    }

    /**
     * Label of the statement.
     */
    protected String label;
    /**
     * Statement being labeled.
     */
    protected Statement statement;
}
