/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents a local variable declaration.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;
import uiuc.Jumbo.Util.*;
import uiuc.Jumbo.Jaemus.*;

public class LocalVariableDeclaration extends Statement {
    /**
     * @param t        Type of the field being declared
     * @param vds      List of variable declarators
     * @param ifs      Flag indicating whether the declaration is final
     */
    public LocalVariableDeclaration( Type t, LinkedList vds, 
                                     boolean isf ) { 
        variable_declarations = vds;
        setParents(vds);
        is_final = isf;

        setType(t);
    }

    public LocalVariableDeclaration( Type t, VariableDeclaration vd, boolean isf) {
        variable_declarations = new LinkedList();
        variable_declarations.add(vd);
        setParents(variable_declarations);
        is_final = isf;
        setType(t);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == type) {
            setType((Type)new_child);
            return true;
        }
        else {
            if (replaceInList(variable_declarations, old_child, new_child)) {
                if (new_child != null)
                    ((VariableDeclaration)new_child).variable.setType(type.copyType());
                return true;
            }
            else {
                return false;
            }
        }
    }

    private void setType(Type t) {
        type = t;
        setParent(type);

        Iterator iter = variable_declarations.iterator();
        while( iter.hasNext() ) {
            ((VariableDeclaration)iter.next()).variable.setType(type.copyType());
        }
    }

    public IR toIR() {
        if (variable_declarations.size() == 1) {
            VariableDeclaration vd = (VariableDeclaration)variable_declarations.get(0);
            return vd.addIR(IR.make("defineVariable", new ArrayMonoList()
                                    .add(IR.make(is_final?Flags.FINAL:0))));
                
        } else {
            MonoList l = new ArrayMonoList();
            for (Iterator i = variable_declarations.iterator(); i.hasNext(); ){
                VariableDeclaration vd = (VariableDeclaration)i.next();
                l.add(vd.addIR(IR.make("defineVariable", new ArrayMonoList()
                                       .add(IR.make(is_final?Flags.FINAL:0)))));
            }
            return IR.make("statements", new ArrayMonoList()
                           .add(IR.make(l)));
        }
    }

    public void toSource() {
        if (variable_declarations.size() == 0) return;

        Iterator iter = variable_declarations.iterator();
        while( iter.hasNext() ) {
            if (is_final) print("final ");
            VariableDeclaration vd = (VariableDeclaration)iter.next();
            vd.variable.type.toSource();
            print(" ");
            vd.toSource();
            if (iter.hasNext()) {
                print(";");
                linebreak();
            }
        }
    }

    /**
     * Type of the field declaration(s)
     */
    protected Type type;
    /**
     * Fields being declared.
     */
    protected LinkedList variable_declarations;
    /**
     * Flag indicating whether this is final or not.
     */
    protected boolean is_final;
}
