/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents a list of modifiers (such as <code>public</code>, 
 * <code>final</code>, etc) common to class/interface declarations, 
 * method/field declarations, and variable declarations.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;

import uiuc.Jumbo.Jaemus.Flags;

public class ModifierList extends Node {
    /**
     * Creates a ModifierList with an initial modifier.
     *
     * @param modifier   Modifier string to be added.
     */
    public ModifierList( String modifier ) { 
        modifiers = new LinkedList();
        modifiers.addLast( modifier );
    }

    /**
     * Adds another modifier to the list.
     *
     * @param modifier   Modifier string to be added.
     */
    public void add( String modifier ) {
        modifiers.addLast( modifier );
    }

    public static void printNone() {
        print("0");
    }

    /**
     * Prints out a list of modifier flags as source strings.
     */
    public void toSource() {
        Iterator iter = modifiers.iterator();
        while( iter.hasNext() ) {
            print( ((String)iter.next()).toLowerCase()+" " );
        }
    }

    public IR toIR() {
        return IR.make(makeModifiers());
    }

    /** Creates an int with the bit pattern of these flags.
    */
    public int makeModifiers() {
        int mod = 0;
        Iterator iter = modifiers.iterator();
        while( iter.hasNext() ) {
            String m = (String)iter.next();
            int f = Flags.fromString(m.toLowerCase());
            if (f == -1)
                throw new Error("Unknown flag "+m+", shouldn't happen");
            mod |= f;
        }
        return mod;
    }

    public boolean contains(String flag) {
        return modifiers.contains(flag);
    }

    public String toString() {
        String s = "";
        Iterator iter = modifiers.iterator();
        while( iter.hasNext() ) {
            s += ((String)iter.next())+" ";
        }
        return s;
    }

    /**
     * List of modifiers.
     */
    protected LinkedList modifiers;
}
