/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Jaemus.*;
import uiuc.Jumbo.Util.*;
import uiuc.Jumbo.Compiler.*;

/**
 * Represents an argument parameter in a method declaration.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class Parameter extends Node {
    /**
     * @param t  parameter type
     * @param v  parameter variable
     * @param f  whether this parameter is final or not.
     */
    public Parameter( Type t, Variable v, boolean f ) { 
        is_final = f;

        type = t;             // have to set type explicitly here;
        setParent(type);      // setType depends on variable existing
        // and vice versa for setVariable
        setVariable(v);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == type) {
            setType((Type)new_child);
            return true;
        }
        else if (old_child == variable) {
            setVariable((Variable)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setType(Type t) {
        type = t;
        setParent(type);
        variable.setType(type.copyType());
    }

    private void setVariable(Variable v) {
        variable = v;
        setParent(variable);
        variable.setType(type.copyType());
    }

    public IR toIR() {
        return IR.make((is_final?Flags.FINAL:0), variable.type.toType(),
                       variable.name.toIRString());
    }

    public void toSource() {
        print(toString());
    }

    public String toString() {
        String result = "";
        if (is_final) result += "final ";
        result += variable.type.toString()+" "+variable.name.toString();
        return result;
    }

    /** 
     * Parameter type.
     */
    protected Type type;

    /**
     * Paramter variable.
     * Contains the type.
     */
    protected Variable variable;

    /**
     * Flag indicating whether this parameter is final.
     */
    protected boolean is_final;
}
