/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * A qualified name that ends up in an expression.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class QualifiedNameExpression extends Expression  {
    /**
     * @param n  the name of the expression
     */
    public QualifiedNameExpression( Identifier n ) { 
        constructor = "QualifiedName";
        name = n;
    }

    public IR toIR() {
        return IR.make("get" + constructor, new ArrayMonoList()
                        .add(name.toIRString()));
    }

    public void toSource() {
        name.toSource();
    }

    public String toString() {
        return name.toString();
    }

    /**
     * Returns a new type object based on this name.
     */
    public Type toType() {
        return new Type( name );
    }

    public Type getType(ClassLoader loader) {
        int firstfield = 0;
        Type t = loader.getVarType(name);
        if (t == null) {
            ClassDefiner cd = getThis();
            String first = name.getField(0).toString();
            t = cd.findFieldType(first);
        } else {
            System.out.println("Variable "+name.getField(0)+" type "+t);
        }
        if (t != null) {
            firstfield = 1;
        } else {
            String classname = "";
            for (int i = 0; i < name.size(); i++) {
                classname += name.getField(i).toString();
                ClassDefiner cd = loader.getClass(classname);
                if (cd != null) {
                    t = new Type(cd.getFQCN());
                    firstfield = i+1;
                    break;
                }
                classname += ".";
            }
        }
        if (firstfield != 0) {
            for (int i = firstfield; i < name.size(); i++) {
                ClassDefiner cd = loader.getClass(t.getName());
                String s = name.getField(i).toString();
                t = cd.findFieldType(s);
                if (t == null) {
                    throw new Error("Null type for "+this+" (first field "+firstfield+")");
                }
            }
        }
        if (t == null) throw new Error("Null type for "+this);
        return t;
    }

    /**
     * The qualified name.
     */
    protected Identifier name;
}
