/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;
import uiuc.Jumbo.Util.*; 

/**
 * Represents a case group within a switch statement.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class SwitchCaseBlock extends Statement {
    /** Creates a switch case block with no contents, only used by
     * AntiquoteSwitchCaseBlock
     */
    protected SwitchCaseBlock( ) {
        labels = null;
        block = null;
    }

    /** 
    * Creates a default case block.
    */
   public SwitchCaseBlock( StatementList b ) {
      labels = null;
      setBlock(b);
   }

   /**
    * @param l   list of case labels
    * @param b   block of the case
    */
   public SwitchCaseBlock( LinkedList l, StatementList b ) { 
      setLabels(l);
      setBlock(b);
   }

   private void setLabels(LinkedList l) {
      labels = l;
      checkLabels();
      setParents(labels);
   }

   private void checkLabels() {
      // check to see if one of the labels is default (e.g., null).
      Iterator iter = labels.iterator();
      while( iter.hasNext() ) {
         if ( iter.next() == null ) {
            // it's a default; clear out the list.
            labels = null;
            break;
         }
      }
   }

   protected boolean replaceUC(Node old_child, Node new_child) {
       if (old_child == block) {
           setBlock((StatementList)new_child);
           return true;
       }
       else if (labels == null) {
           if (old_child == null) {
               LinkedList l = new LinkedList();
               l.add(new_child);
               setLabels(l);
               return true;
           }
           else {
               return false;
           }
       }
       else {
           return replaceInList(labels, old_child, new_child);
       }
   }

   private void setBlock(StatementList b) {
      block = b;
      setParent(block);
   }

   /**
    * Indicates whether this is a default switch case.
    *
    * @return true if this case group is a default, false otherwise.
    */
   public boolean isDefault() {
      return labels == null;
   }

    public IR toIR() { 
        if (isDefault())
            return block.toIR();
        else {
            MonoList l = new ArrayMonoList();
            for (Iterator i = labels.iterator(); i.hasNext(); ) {
                l.add(((Node)i.next()).toIR());
            }
            return IR.make(l, (IRCode)block.toIR());
        }
    }

    public void toSource() {
        if (isDefault()) {
            print("default: ");
        } else {
            Iterator iter = labels.iterator();
            while( iter.hasNext() ) {
                print("case ");
                ((Node)iter.next()).toSource();
                print(": ");
            }
        }
        block.toSource();
    }

   /**
    * Cases.
    */
   protected LinkedList labels;
   /**
    * Block.
    */
   protected StatementList block;
}
