/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

/**
 * Represents a try-catch-finally statement.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class TryCatchStatement extends Statement {
   /**
    * @param tb     try block
    * @param cl     optional list of catch blocks
    * @param f      finally block
    */
   public TryCatchStatement( StatementList tb, LinkedList cl,
                             StatementList f ) { 
      try_block = tb;
      catches_list = cl;
      finally_block = f;
   }

    public IR toIR() {
        if (finally_block != null) {
            return IR.make("handlersWithFinally", new ArrayMonoList()
                           .add(try_block.toIR())
                           .add(toIRList(catches_list))
                           .add(finally_block.toIR()));
        } else {
            return IR.make("handlers", new ArrayMonoList()
                           .add(try_block.toIR())
                           .add(toIRList(catches_list)));
        }
    }

    public void toSource() {
        print("try ");
        try_block.toSource();
        printSourceList(catches_list);
        if (finally_block != null) {
            print("finally ");
            finally_block.toSource();
        }
    }
    public boolean needsSemicolon() { return false; }

   /**
    * Try block.
    */
   protected StatementList try_block;
   /**
    * List of catch blocks.
    */
   protected LinkedList catches_list;
   /**
    * Finally block.
    */
   protected StatementList finally_block;
}
