/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

/**
 * Abstract base class for any type (class/interface) declaration.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public abstract class TypeDeclaration extends Statement {
    /**
     * @param tname  name of the type being declared
     * @param mods   list of modifier flags
     */
    public TypeDeclaration( ModifierList mods, Identifier tname ) { 
        modifiers = mods;
        setName(tname);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == name) {
            setName((Identifier)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setName(Identifier tname) {
        name = tname;
        setParent(name);
    }

    /** Returns the name of this class or interface.
     */
    public String getName() {
        return name.toString();
    }

    /** Returns the fully qualified class name for this class or interface.
     */
    public String getFQCN() {
        if (fqcn != null) return fqcn;
        Node p = this;
        if (p != null) {
            p = p.parent;
            if (p == null) fqcn = getName();
            else {
                // Find package statement -- must be first
                if (p instanceof StatementList) {
                    StatementList sl = (StatementList)p;
                    Node n = (Node) sl.statements.get(0);
                    if (n instanceof PackageDeclaration) {
                        PackageDeclaration pd = (PackageDeclaration)n;
                        fqcn = pd.package_name.toString()+"."+getName();
                    }
                }
            }
        }
        return fqcn;
    }

    /** Return true if this is a final class.
     */
    public boolean isFinal() {
        return modifiers.contains("FINAL");
    }

    /**
     * Name of the type being declared
     */
    protected Identifier name;
    /**
     * List of modifier flags
     */
    protected ModifierList modifiers;
    /**
     * Fully qualified name for this class.
     * If null, we haven't looked for the package yet.
     */
    protected String fqcn;
}
